/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * All the GNOME controls are derived from GtkWidget so we provide a set of
 * common handlers that can be used for all controls.
 */


#include <gtk/gtkwidget.h>

#include "gvpmodint.h"


static void handleDestroy(GtkObject *,gpointer);


/*
 * Get the background color.
 */

int gvpGetBackground(void *self,pymodColorDef *cd)
{
	GtkStyle *gsp;
	GdkColor *gcp;

	gsp = gtk_widget_get_style((GtkWidget *)self);

	gcp = &gsp -> bg[GTK_STATE_NORMAL];

	cd -> red = gcp -> red;
	cd -> green = gcp -> green;
	cd -> blue = gcp -> blue;
	cd -> bitsPerColor = 16;

	return 0;
}


/*
 * Destroy a widget.
 */

void gvpDtor(void *self)
{
	gtk_widget_destroy((GtkWidget *)self);
}


/*
 * Carry out initialisation common to all widgets.
 */

void gvpWidgetInit(GtkWidget *w,gpointer context)
{
	gtk_signal_connect(GTK_OBJECT(w),"destroy",GTK_SIGNAL_FUNC(handleDestroy),context);
}


/*
 * Handle the destruction of a widget.
 */

static void handleDestroy(GtkObject *self,gpointer data)
{
	/* The first element of any context must be a PyObject *. */

	pymodWidgetDeleted(*(PyObject **)data);

	Py_Free((ANY *)data);
}
