/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This handles the GNOME implementation of the Message class.
 */


#include <gnome.h>

#include "gvpmodint.h"


static gint handleDialog(GtkWidget *,void *,char *);


/*
 * The static method error.
 */

void gvpMessageError(char *text,void *parent,char *title)
{
	handleDialog(gnome_error_dialog(text),parent,title);
}


/*
 * The static method sorry.
 */

void gvpMessageSorry(char *text,void *parent,char *title)
{
	/* Make the closest thing we can to a GNOME sorry dialog. */

	if (title == NULL)
		title = _("Sorry");

	handleDialog(gnome_warning_dialog(text),parent,title);
}


/*
 * The static method information.
 */

void gvpMessageInformation(char *text,void *parent,char *title)
{
	handleDialog(gnome_ok_dialog(text),parent,title);
}


/*
 * The static method question.
 */

int gvpMessageQuestion(char *text,void *parent,char *title,char *notext, char *yestext)
{
	gint gbutt;

	if (yestext == NULL)
		yestext = GNOME_STOCK_BUTTON_YES;

	if (notext == NULL)
		notext = GNOME_STOCK_BUTTON_NO;

	gbutt = handleDialog(gnome_message_box_new(text,GNOME_MESSAGE_BOX_QUESTION,yestext,notext,NULL),parent,title);

	return (gbutt == GNOME_YES ? PYMOD_DIALOG_BUTTON_YES : PYMOD_DIALOG_BUTTON_NO);
}


/*
 * The static method warning.
 */

int gvpMessageWarning(char *text,void *parent,char *title,int butts,char *continuetext,char *notext,char *yestext)
{
	gchar *buttin[3];
	gint gbutt;
	int buttout[4];

	if (yestext == NULL)
		yestext = GNOME_STOCK_BUTTON_YES;

	if (notext == NULL)
		notext = GNOME_STOCK_BUTTON_NO;

	if (continuetext == NULL)
		continuetext = _("Continue");

	/* Create the right buttons. */

	buttin[0] = buttin[1] = buttin[2] = NULL;

	switch (butts)
	{
	case PYMOD_DIALOG_BUTTON_YES|PYMOD_DIALOG_BUTTON_NO:
		buttin[0] = yestext;
		buttin[1] = notext;
		buttout[0] = PYMOD_DIALOG_BUTTON_NO;
		buttout[1] = PYMOD_DIALOG_BUTTON_YES;
		buttout[2] = PYMOD_DIALOG_BUTTON_NO;
		break;

	case PYMOD_DIALOG_BUTTON_CONTINUE|PYMOD_DIALOG_BUTTON_CANCEL:
		buttin[0] = continuetext;
		buttin[1] = GNOME_STOCK_BUTTON_CANCEL;
		buttout[0] = PYMOD_DIALOG_BUTTON_CANCEL;
		buttout[1] = PYMOD_DIALOG_BUTTON_CONTINUE;
		buttout[2] = PYMOD_DIALOG_BUTTON_CANCEL;
		break;

	case PYMOD_DIALOG_BUTTON_YES|PYMOD_DIALOG_BUTTON_NO|PYMOD_DIALOG_BUTTON_CANCEL:
		buttin[0] = yestext;
		buttin[1] = notext;
		buttin[2] = GNOME_STOCK_BUTTON_CANCEL;
		buttout[0] = PYMOD_DIALOG_BUTTON_CANCEL;
		buttout[1] = PYMOD_DIALOG_BUTTON_YES;
		buttout[2] = PYMOD_DIALOG_BUTTON_NO;
		buttout[3] = PYMOD_DIALOG_BUTTON_CANCEL;
		break;
	}

	gbutt = handleDialog(gnome_message_box_new(text,GNOME_MESSAGE_BOX_WARNING,buttin[0],buttin[1],buttin[2],NULL),parent,title);

	/* Sanity check. */

	if (gbutt < -1 || gbutt > 2)
		gbutt = -1;

	return buttout[gbutt + 1];
}


/*
 * Do most of the work of handling a dialog.  Return the button clicked.
 */

static gint handleDialog(GtkWidget *dlg,void *ptr,char *title)
{
	GtkWidget *parent;

	if (title != NULL)
		gtk_window_set_title(GTK_WINDOW(dlg),title);

	gvpDialogSetParent(dlg,ptr);

	return gnome_dialog_run_and_close(GNOME_DIALOG(dlg));
}
