/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This is the main module of the GNOME version of the VP shell.
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <gdk_imlib.h>

#include <gvpinterpreter.h>

#include "gvpshell.h"


static gint sessionDie(GnomeClient *,gpointer);


int main(int argc,char **argv)
{
	char **args, *appid = "gvpshell";
	int nrargs;
	poptContext pctx;
	GtkWidget *w, *darea;
	GnomeClient *client;
	GtkObject *vpinterp;

	struct poptOption options[] = {
		{NULL}
	};

	/* Initialise. */

	if (gnome_init_with_popt_table(appid,VERSION,argc,argv,options,0,&pctx))
		exit(1);

	gdk_imlib_init();

	/* Count the unused arguments. */

	if ((args = (char **)poptGetArgs(pctx)) != NULL)
		for (nrargs = 0; args[nrargs] != NULL; ++nrargs)
			;
	else
		nrargs = 0;

	/* Session management. */

	client = gnome_master_client();

	gtk_signal_connect(GTK_OBJECT(client),"die",GTK_SIGNAL_FUNC(sessionDie),NULL);

	/* Create the main window. */

	w = gvpshell_new(appid,&darea);
	gtk_window_set_default_size(GTK_WINDOW(w),250,250);
	gtk_widget_show_all(w);

	/*
	 * Create an interpreter, set the default "drawing area" and pass down
	 * any remaining arguments.
	 */

	vpinterp = vp_interpreter_new(darea);

	/* Connect up the error handlers. */

	gtk_signal_connect(vpinterp,"script_failed",
			   GTK_SIGNAL_FUNC(gvpshell_script_failed_slot),w);

	gtk_signal_connect(vpinterp,"internal_error",
			   GTK_SIGNAL_FUNC(gvpshell_internal_error_slot),w);

	/* Set the arguments. */

	vp_interpreter_set_argv(vpinterp,nrargs,args,NULL);

	/*
	 * The first argument is the name of a script.  At the moment we don't
	 * support reading from stdin if there were no arguments.
	 */

	if (nrargs >= 1)
		vp_interpreter_execute(vpinterp,args[0]);

	poptFreeContext(pctx);

	gtk_main();

	return 0;
}


/*
 * Handle the session termination.
 */

static gint sessionDie(GnomeClient *client,gpointer data)
{
	gtk_main_quit();

	return TRUE;
}
