// Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
//
// The implementation of the VPInterpreter class.


#include <kapp.h>
#include <klocale.h>
#include <kmessagebox.h>

#include "pylib.h"
#include "kvpvalue.h"

#include "kvpinterpreter.moc"


VPInterpreter *VPInterpreter::main = 0L;


extern "C" void VPErrorHandler(const char *,void *);


// The ctor.  defAppWindow is the default application window to make available
// to all scripts run in this interpreter.

VPInterpreter::VPInterpreter(QWidget *defAppWindow)
{
	m_pre1code = "import vp, libkvp";
	m_pre2code = 0;
	m_postcode = 0;

	m_context = pylibInit(VPErrorHandler,this);

	addApplicationWindow(defAppWindow);

	if (!main)
		main = this;
}


// The dtor.

VPInterpreter::~VPInterpreter()
{
	pylibDone(m_context,m_postcode);

	if (main == this)
		main = 0;
}


// Execute a script in the interpreter.  script is the name of a file
// containing the script.

void VPInterpreter::execute(char *script)
{
	const char *emess;

	emess = pylibExec(m_context,script,m_pre1code,m_pre2code);

	if (emess)
		emit scriptFailed(script,emess);
}


// Tell the Python library about an application window.  name is the name a
// script will use to refer to the window or 0 if this is the default window.
// appWindow is the application window.

void VPInterpreter::addApplicationWindow(QWidget *appWindow,const char *name)
{
	pylibAddAppWindow(m_context,appWindow,name);
}


// Tell the Python library that an application window has been removed.  name
// is the name a script will use to refer to the window.  The default window
// cannot be removed, only changed.

void VPInterpreter::removeApplicationWindow(QWidget *appWindow)
{
	pylibRemoveAppWindow(m_context,appWindow);
}


// Get the context for an interpreter.

struct pylibContext *VPInterpreter::getContext(VPInterpreter *interp)
{
	if (!interp)
		interp = main;

	return (interp ? interp -> m_context : 0L);
}


// Set an application window's argv property.

void VPInterpreter::setArgv(int argc,char **argv,QWidget *appWindow)
{
	pylibSetAppWindowArgv(m_context,appWindow,argc,argv);
}


// Set the signals for all application windows.

void VPInterpreter::setSignals(char **signames)
{
	pylibSetAppWindowSignals(m_context,signames);
}


// Emit an application window signal.

void VPInterpreter::emitSignal(char *signame,VPValue *val,QWidget *appWindow)
{
	PyObject *pyval = (val ? val -> getPyObj() : 0);

	pylibEmitAppWindowSignal(m_context,appWindow,signame,pyval);
}


// Emit an internalError signal on behalf of the error handler.

void VPInterpreter::emitInternalError(const char *message)
{
	emit internalError(message);
}


// This is the error handler called by the Python library whenever there has
// been a significant internal error.  emess describes the nature of the error.

extern "C" void VPErrorHandler(const char *emess,void *tkData)
{
	((VPInterpreter *)tkData) -> emitInternalError(emess);
}
