// Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
//
// The implementation of the VPValue, VPValueDict and VPValueList classes.


#include "pylib.h"
#include "kvpinterpreter.h"

#include "kvpvalue.h"


// Create a string value.

VPValue::VPValue(const char *strval,VPInterpreter *interp)
{
	setContext(interp);

	m_pyobj = pylibValueCtorString(getContext(),strval);
}


// Create a long value.

VPValue::VPValue(long longval,VPInterpreter *interp)
{
	setContext(interp);

	m_pyobj = pylibValueCtorLong(getContext(),longval);
}


// Create a double value.

VPValue::VPValue(double doubleval,VPInterpreter *interp)
{
	setContext(interp);

	m_pyobj = pylibValueCtorDouble(getContext(),doubleval);
}


VPValue::~VPValue()
{
	pylibValueDtor(getContext(),m_pyobj);
}


// Create a dictionary value.

VPValueDict::VPValueDict(VPInterpreter *interp) : VPValue(interp)
{
	m_pyobj = pylibValueDictCtor(getContext());
}


// Set a key/value pair in a dictionary.

void VPValueDict::set(VPValue *key,VPValue *val)
{
	// Make sure the contexts are the same.

	if (getContext() == key -> getContext() && getContext() == val -> getContext())
		pylibValueDictSet(getContext(),m_pyobj,key -> getPyObj(),val -> getPyObj());
}


// Create a list value.

VPValueList::VPValueList(VPInterpreter *interp) : VPValue(interp)
{
	m_pyobj = pylibValueListCtor(getContext());
}


// Append a value to a list.

void VPValueList::append(VPValue *val)
{
	// Make sure the contexts are the same.

	if (getContext() == val -> getContext())
		pylibValueListAppend(getContext(),m_pyobj,val -> getPyObj());
}
