// Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
//
// This handles the KDE implementation of the PieChart and Graph classes.


#include <qwidget.h>
#include <qpixmap.h>

#include "kvpmodint.h"


// We need to sub-class the Qt widget to add some extras.

class kvpChart : public QWidget
{
public:
	kvpChart(PyObject *,QWidget *);
	~kvpChart();

	void plot(const uchar *image,unsigned image_size)
	{
		qpix.loadFromData(image,image_size);
		setFixedSize(qpix.width(),qpix.height());
	}

protected:
	QSize sizeHint() const;
	QSizePolicy sizePolicy() const;
	void paintEvent(QPaintEvent *);
 
private:
	QPixmap qpix;			// The image in Qt format.

	PyObject *pyobj;		// The corresponding Python object.
 
	kvpChart(const kvpChart &);
	kvpChart &operator=(const kvpChart &);
};


// The kvpChart ctor.
 
kvpChart::kvpChart(PyObject *po,QWidget *parent) :
        QWidget(parent)
{
	pyobj = po;
}
 
 
// The kvpChart dtor.
 
kvpChart::~kvpChart()
{
	pymodWidgetDeleted(pyobj);
}


// Paint the kvpChart.

void kvpChart::paintEvent(QPaintEvent *)
{
	bitBlt(this,0,0,&qpix,0,0,-1,-1,CopyROP);
}


// Return the preferred size of the kvpChart.

QSize kvpChart::sizeHint() const
{
	return QSize(minimumSize().width(),minimumSize().height());
}


// Return the size policy of the kvpChart.

QSizePolicy kvpChart::sizePolicy() const
{
	return QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
}


// The dtor.

extern "C" void kvpChartDtor(void *ptr)
{
	delete (kvpChart *)ptr;
}


// The ctor.

extern "C" void *kvpChartCtor(PyObject *po,void *parent,int width,int height)
{
	kvpChart *kc = new kvpChart(po,(QWidget *)parent);

	if (kc)
		kc -> setFixedSize(width,height);

	return kc;
}


// The plot method.

extern "C" void kvpChartPlot(void *self,void *image,int image_size)
{
	((kvpChart *)self) -> plot((const uchar *)image,image_size);
}
