// Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
//
// This handles the KDE implementation of the Dialog class.


#include <kdialogbase.h>
#include <klocale.h>
#include <qhbox.h>

#include "kvpmodint.h"


// We need to sub-class the KDE dialog to add some extras.

class kvpDialog : public KDialogBase
{
public:
	kvpDialog(PyObject *,QWidget *,char *,int,ButtonCode,bool);
	~kvpDialog();

private:
	void slotOk();
	void slotCancel();
	void slotUser1();
	void slotUser2();
	void slotUser3();
	void closeEvent(QCloseEvent *);

	PyObject *pyobj;		// The corresponding Python object.

	kvpDialog(const kvpDialog &);
	kvpDialog &operator=(const kvpDialog &);
};


// The kvpDialog ctor.

kvpDialog::kvpDialog(PyObject *po,QWidget *parent,char *title,int buttons,ButtonCode defaultButton,bool modal) :
	KDialogBase(parent,0,modal,title,buttons,defaultButton)
{
	pyobj = po;
}


// The kvpDialog dtor.

kvpDialog::~kvpDialog()
{
	pymodWidgetDeleted(pyobj);
}


// Called when the Ok button is clicked.

void kvpDialog::slotOk()
{
	pymodSignalDialogClicked(pyobj,PYMOD_DIALOG_BUTTON_OK);
}


// Called when the Cancel button is clicked.

void kvpDialog::slotCancel()
{
	pymodSignalDialogClicked(pyobj,PYMOD_DIALOG_BUTTON_CANCEL);
}


// Called when the Continue button is clicked.

void kvpDialog::slotUser3()
{
	pymodSignalDialogClicked(pyobj,PYMOD_DIALOG_BUTTON_CONTINUE);
}


// Called when the Yes button is clicked.

void kvpDialog::slotUser2()
{
	pymodSignalDialogClicked(pyobj,PYMOD_DIALOG_BUTTON_YES);
}


// Called when the No button is clicked.

void kvpDialog::slotUser1()
{
	pymodSignalDialogClicked(pyobj,PYMOD_DIALOG_BUTTON_NO);
}


// Called when the close box is clicked.

void kvpDialog::closeEvent(QCloseEvent *)
{
	pymodSignalDialogClicked(pyobj,0);
}


// The dtor.

extern "C" void kvpDialogDtor(void *ptr)
{
	delete (kvpDialog *)ptr;
}


// The ctor.

extern "C" void *kvpDialogCtor(PyObject *po,void *parent,char *title,int buttons,char *continueText,int defaultButton,int modality,char *noText,char *yesText)
{
	int bmask = 0;
	KDialogBase::ButtonCode kdefbutt;
	kvpDialog *kd;

	if (buttons & PYMOD_DIALOG_BUTTON_OK)
		bmask |= KDialogBase::Ok;

	if (buttons & PYMOD_DIALOG_BUTTON_CANCEL)
		bmask |= KDialogBase::Cancel;

	if (buttons & PYMOD_DIALOG_BUTTON_CONTINUE)
		bmask |= KDialogBase::User3;

	if (buttons & PYMOD_DIALOG_BUTTON_YES)
		bmask |= KDialogBase::User2;

	if (buttons & PYMOD_DIALOG_BUTTON_NO)
		bmask |= KDialogBase::User1;

	switch (defaultButton)
	{
	case PYMOD_DIALOG_BUTTON_OK:
		kdefbutt = KDialogBase::Ok;
		break;

	case PYMOD_DIALOG_BUTTON_CANCEL:
		kdefbutt = KDialogBase::Cancel;
		break;

	case PYMOD_DIALOG_BUTTON_CONTINUE:
		kdefbutt = KDialogBase::User3;
		break;

	case PYMOD_DIALOG_BUTTON_YES:
		kdefbutt = KDialogBase::User2;
		break;

	case PYMOD_DIALOG_BUTTON_NO:
		kdefbutt = KDialogBase::User1;
		break;
	}

	kd = new kvpDialog(po,(QWidget *)parent,title,bmask,kdefbutt,
				(modality == PYMOD_DIALOG_MODALITY_MODAL));

	/* Set default button texts. */

	if (kd != NULL)
	{
		if (buttons & PYMOD_DIALOG_BUTTON_CONTINUE)
			kd -> setButtonText(KDialogBase::User3,
						(continueText != NULL ?
						 QString(continueText) :
						 i18n("C&ontinue")));

		if (buttons & PYMOD_DIALOG_BUTTON_YES)
			kd -> setButtonText(KDialogBase::User2,
						(yesText != NULL ?
						 QString(yesText) :
						 i18n("&Yes")));

		if (buttons & PYMOD_DIALOG_BUTTON_NO)
			kd -> setButtonText(KDialogBase::User1,
						(noText != NULL ?
						 QString(noText) :
						 i18n("&No")));
	}

	return kd;
}


// Add a child.

extern "C" void kvpDialogAddChild(void *self,void *child)
{
	((KDialogBase *)self) -> setMainWidget((QWidget *)child);
}


// The method hide.

extern "C" void kvpDialogHide(void *self)
{
	((kvpDialog *)self) -> hide();
}


// The method show.

extern "C" void kvpDialogShow(void *self)
{
	((kvpDialog *)self) -> show();
}


// Set the title property.

extern "C" int kvpDialogSetTitle(void *self,char *title)
{
	((kvpDialog *)self) -> setCaption(title);

	return 0;
}


// Set the continueText property.

extern "C" int kvpDialogSetContinueText(void *self,char *continueText)
{
	((kvpDialog *)self) -> setButtonText(KDialogBase::User3,continueText);

	return 0;
}


// Set the noText property.

extern "C" int kvpDialogSetNoText(void *self,char *noText)
{
	((kvpDialog *)self) -> setButtonText(KDialogBase::User1,noText);

	return 0;
}


// Set the yesText property.

extern "C" int kvpDialogSetYesText(void *self,char *yesText)
{
	((kvpDialog *)self) -> setButtonText(KDialogBase::User2,yesText);

	return 0;
}
