// Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
//
// This is the main module of the KDE version of the VP shell.


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kcmdlineargs.h>

#include <kvpinterpreter.h>

#include "kvpshell.h"


int main(int argc,char **argv)
{
	KAboutData about("kvpshell",I18N_NOOP("KDE Visual Python Shell"),VERSION);
	KVPShell *w;
	VPInterpreter *vpinterp;

	// Handle the command line.

	static KCmdLineOptions options[] = {
		{"+[script]", I18N_NOOP("The name of a script file"), 0},
		{"+[script-arg]", I18N_NOOP("An argument to the script"), 0},
		{0, 0, 0}
	};

	KCmdLineArgs::init(argc,argv,&about);
	KCmdLineArgs::addCmdLineOptions(options);
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

	char *newargv[args -> count() + 1];

	for (int i = 0; i < args -> count(); ++i)
		newargv[i] = (char *)args -> arg(i);

	newargv[args -> count()] = NULL;

	KApplication app;

	// See if we are restoring a session.

	if (app.isRestored())
	{
		// Restore the session and remember one of the restored
		// windows - probably the only one.

		for (int n = 1; KTMainWindow::canBeRestored(n); ++n)
		{
			w = new KVPShell;
			w -> restore(n);
		}
	}
	else
	{
		w = new KVPShell;
		w -> show();
	}

	// Create an interpreter, set the default "drawing area" and pass down
	// any remaining arguments.

	vpinterp = new VPInterpreter(w -> view());

	// Connect up the error handlers.

	QObject::connect(vpinterp,SIGNAL(scriptFailed(const char *,const char *)),
			 w,SLOT(scriptFailed(const char *,const char *)));

	QObject::connect(vpinterp,SIGNAL(internalError(const char *)),
			 w,SLOT(internalError(const char *)));

	// Set the arguments.

	vpinterp -> setArgv(args -> count(),newargv);

	// The first argument is the name of a script.  At the moment we don't
	// support reading from stdin if there were no arguments.
 
	if (args -> count() >= 1)
		vpinterp -> execute(newargv[0]);

	args -> clear();

	return app.exec();
}
