/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module handles the execution of a single script.
 */


#include <stdio.h>

#include "pylibint.h"


static int doneFirstScript = 0;		/* Set if the module is initialised. */


/*
 * Return non-zero if the Python module has been initialised.
 */

int pylibDoneFirstScript(void)
{
	return doneFirstScript;
}


/*
 * Execute a Python script, including all the pre- and post-scripts.
 */

const char *pylibExec(struct pylibContext *context,char *scriptname,char *pre1code,char *pre2code)
{
	const char *emess;
	FILE *fp;

	pylibRestoreContext(context);

	/* Only execute the pre-code for the first script. */

	if ((fp = fopen(scriptname,"r")) == NULL)
		emess = "Unable to open file.";
	else if (!context -> ranScript && pre1code && PyRun_SimpleString(pre1code) != 0)
		emess = "A Python error occured while executing the first pre-code.";
	else if (!context -> ranScript && pre2code && PyRun_SimpleString(pre2code) != 0)
		emess = "A Python error occured while executing the second pre-code.";
	else if (PyRun_SimpleFile(fp,scriptname) != 0)
		emess = "A Python error occured while executing the script.";
	else
	{
		emess = NULL;
		context -> ranScript = 1;
	}

	doneFirstScript = 1;

	if (fp != NULL)
		fclose(fp);

	pylibClearContext();

	return emess;
}
