/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This header files defines the interface between the GUI independent part of
 * the application API to VP and the GUI independent part of the Python module.
 */


#ifndef _PYLIBMOD_H
#define	_PYLIBMOD_H


#include <Python.h>


#ifdef __cplusplus
extern "C" {
#endif


/*
 * pymod code can call pylib functions because the toolkit API library will
 * have been loaded if the VP implementation library is loaded.  However, the
 * reverse isn't true.  Therefore pylib (rather than pymod) needs to do some
 * changes to Application Windows.  This means that we need to expose some of
 * pymod's data structures to pylib.
 */

typedef struct _classDef classDef;
typedef struct _connectionDef connectionDef;


/* This defines a signal. */

typedef struct {
	char			*name;		/* The signal name. */
	PyObject		*nameobj;	/* The name as an object. */
} signalDef;


/*
 * This defines the data that is common to all wrapped toolkit objects.  One of
 * these must appear at the start (after the standard Python part) of all
 * wrapped object types.
 */

typedef struct {
	int			flags;		/* The wrapper flags. */
	classDef		*cd;		/* The class meta-data. */
	PyObject		*instance;	/* The class instance. */
	void			*tkPtr;		/* The toolkit pointer. */
	connectionDef		*connections;	/* What it is connected to. */
} pymodDataBase;


/*
 * This defines a wrapped toolkit object.  It is never actually created because
 * each non-abstract object will define a more specific type.
 */
 
typedef struct {
	PyObject_HEAD
	pymodDataBase		base;		/* The common data. */
} pymodPyBase;


/* This defines a named application window. */
 
typedef struct _pylibAppWindow {
	char			*name;		/* Name or NULL. */
	void			*window;	/* The window. */
	PyObject		*argv;		/* The argv property. */
	pymodPyBase		*bw;		/* The control, if any. */
	struct _pylibAppWindow	*next;		/* Next in the list. */
} pylibAppWindow;


struct pylibContext;


struct pylibContext *pylibGetContext(void);
struct pylibContext *pylibGetAndClearContext(void);
void pylibClearContext(void);
void pylibRestoreContext(struct pylibContext *);
void pylibException(void);
pylibAppWindow *pylibGetAppWindows(void);
signalDef **pylibGetAppWindowSignals(void);
void pylibSetSignalEmitter(int (*)(pymodPyBase *,signalDef *,char *,...));


#ifdef __cplusplus
}
#endif

#endif
