/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module handles application values.
 */


#include "pylibint.h"


/*
 * The ctor for an application string value.
 */

PyObject *pylibValueCtorString(struct pylibContext *context,const char *strval)
{
	PyObject *obj;

	pylibRestoreContext(context);

	if ((obj = PyString_FromString(strval)) == NULL)
		pylibException();

	pylibClearContext();
}


/*
 * The ctor for an application long value.
 */

PyObject *pylibValueCtorLong(struct pylibContext *context,long longval)
{
	PyObject *obj;

	pylibRestoreContext(context);

	if ((obj = PyInt_FromLong(longval)) == NULL)
		pylibException();

	pylibClearContext();
}


/*
 * The ctor for an application double value.
 */

PyObject *pylibValueCtorDouble(struct pylibContext *context,double doubleval)
{
	PyObject *obj;

	pylibRestoreContext(context);

	if ((obj = PyFloat_FromDouble(doubleval)) == NULL)
		pylibException();

	pylibClearContext();
}


/*
 * The dtor for an application value.
 */

void pylibValueDtor(struct pylibContext *context,PyObject *obj)
{
	pylibRestoreContext(context);

	Py_XDECREF(obj);

	pylibClearContext();
}


/*
 * The ctor for an application dictionary value.
 */

PyObject *pylibValueDictCtor(struct pylibContext *context)
{
	PyObject *obj;

	pylibRestoreContext(context);

	if ((obj = PyDict_New()) == NULL)
		pylibException();

	pylibClearContext();
}


/*
 * Set a key/value pair in an application dictionary.
 */

void pylibValueDictSet(struct pylibContext *context,PyObject *dict,PyObject *key,PyObject *val)
{
	pylibRestoreContext(context);

	if (PyDict_SetItem(dict,key,val) < 0)
		pylibException();

	pylibClearContext();
}


/*
 * The ctor for an application list value.
 */

PyObject *pylibValueListCtor(struct pylibContext *context)
{
	PyObject *obj;

	pylibRestoreContext(context);

	if ((obj = PyList_New(0)) == NULL)
		pylibException();

	pylibClearContext();
}


/*
 * Append a value to an application list.
 */

void pylibValueListAppend(struct pylibContext *context,PyObject *list,PyObject *val)
{
	pylibRestoreContext(context);

	if (PyList_Append(list,val) < 0)
		pylibException();

	pylibClearContext();
}
