/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the Chart class.
 */


#include <stddef.h>

#include "pymodint.h"
#include "gdchart.h"


/*****************************************************************************/
/* Describe the Chart class.                                                */
/*****************************************************************************/

/* The properties of this class. */

propDef pymodPropChartHeight = {
	"height",
	uintProp,
	offsetof(pymodPyChart,chart.height),
};
 
propDef pymodPropChartLabelColor = {
	"labelColor",
	instanceProp,
	offsetof(pymodPyChart,chart.labelColor),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	NULL,
	&pymodClassColor,
};

propDef pymodPropChartTitle = {
	"title",
	stringProp,
	offsetof(pymodPyChart,chart.title),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
};

propDef pymodPropChartTitleColor = {
	"titleColor",
	instanceProp,
	offsetof(pymodPyChart,chart.titleColor),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	NULL,
	&pymodClassColor,
};

propDef pymodPropChartWidth = {
	"width",
	uintProp,
	offsetof(pymodPyChart,chart.width),
};

static classPropDef chartProperties[] = {
	{&pymodPropChartHeight, },
	{&pymodPropChartLabelColor, },
	{&pymodPropChartTitle, },
	{&pymodPropChartTitleColor, },
	{&pymodPropChartWidth, },
	{NULL}
};


/* The class definition. */
 
classDef pymodClassChart = {
	"Chart",
	chartProperties,
};


/*
 * Convert a VP color to the format used by the chart library.
 */

unsigned long pymodChartColor(pymodPyColor *cw)
{
	unsigned long red, green, blue;

	if (cw == NULL)
		return GDC_DFLTCOLOR;

	red = cw -> color.color.red >> 8;
	green = cw -> color.color.green >> 8;
	blue = cw -> color.color.blue >> 8;

	return ((red << 16) | (green << 8) | blue);
}
