/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the Container class.
 */


#include <stddef.h>

#include "pymodint.h"

 
/*****************************************************************************/
/* Describe the Container class.                                             */
/*****************************************************************************/
 
/* The properties of this class. */

propDef pymodPropContainerChildren = {
	"children",
	instanceProp,
	offsetof(pymodPyContainer,container.children),
	PROP_FLAG_LIST|PROP_FLAG_NOT_CTOR,
	{0, },
	{0, },
	NULL,
	&pymodClassControl,
};
 
static classPropDef containerProperties[] = {
	{&pymodPropContainerChildren, },
	{NULL}
};
 
/* The class definition. */
 
classDef pymodClassContainer = {
        "Container",
	containerProperties,
};


/*
 * Do the first part of adding a Control to a Container.
 */

int pymodContainerAddChild1(pymodPyBase *self)
{
	pymodDataControl *child = &((pymodPyControl *)self) -> control;
	pymodDataContainer *parent = &child -> parent -> container;

	/* Validate any layout information. */

	if (parent -> control.base.cd == &pymodClassGrid)
	{
		char *bad;

		if (child -> row < 0)
			bad = pymodPropControlRow.name;
		else if (child -> column < 0)
			bad = pymodPropControlColumn.name;
		else
			bad = NULL;

		if (bad != NULL)
		{
			PyErr_Format(PyExc_TypeError,"The %s property must be set for children of instances of %s",bad,pymodClassGrid.name);

			return -1;
		}
	}

	/* Create the list for the children. */

	if (parent -> children == NULL && (parent -> children = PyList_New(0)) == NULL)
		return -1;

	/* There can be any number of TopLevel children. */

	if (self -> base.cd != &pymodClassTopLevel)
	{
		/*
		 * ApplicationWindows and Dialogs may only have one child that
		 * is anything other than a TopLevel.
		 */

		if (parent -> control.base.cd == &pymodClassApplicationWindow ||
		    parent -> control.base.cd == &pymodClassDialog)
		{
			int i;
 
			for (i = 0; i < PyList_GET_SIZE(parent -> children); ++i)
			{
				pymodPyBase *sib = (pymodPyBase *)PyList_GET_ITEM(parent -> children,i);
 
				if (sib -> base.cd != &pymodClassTopLevel)
				{
					PyErr_Format(PyExc_TypeError,"Instances of class %s can only have one child that isn't a %s",parent -> control.base.cd -> name,pymodClassTopLevel.name);
					return -1;
				}
			}
		}
	}

	/*
	 * Append the child's real instance to the parent's list of children so
	 * that the instances reference count is increased.
	 */

	if (PyList_Append(parent -> children,child -> base.instance) < 0)
		return -1;

	/* We actually store the wrapped object in the list. */

	PyList_SET_ITEM(parent -> children,PyList_GET_SIZE(parent -> children) - 1,(PyObject *)self);

	return 0;
}


/*
 * Do the second part of adding a Control to a Container.
 */

void pymodContainerAddChild2(pymodPyBase *self)
{
	pymodDataControl *child = &((pymodPyControl *)self) -> control;
	pymodDataContainer *parent = &child -> parent -> container;

	/* Tell the toolkit about the child if it wants to know. */

	if (parent -> control.base.cd == &pymodClassApplicationWindow &&
	    ((pymodApplicationWindowDrivers *)pymodClassApplicationWindow.drivers) -> addChild)
		(*((pymodApplicationWindowDrivers *)pymodClassApplicationWindow.drivers) -> addChild)(
			parent -> control.base.tkPtr,
			child -> base.tkPtr);
	else if (parent -> control.base.cd == &pymodClassDialog &&
	    ((pymodDialogDrivers *)pymodClassDialog.drivers) -> addChild)
		(*((pymodDialogDrivers *)pymodClassDialog.drivers) -> addChild)(
			parent -> control.base.tkPtr,
			child -> base.tkPtr);
	else if (parent -> control.base.cd == &pymodClassGrid &&
	    ((pymodGridDrivers *)pymodClassGrid.drivers) -> addChild)
		(*((pymodGridDrivers *)pymodClassGrid.drivers) -> addChild)(
			parent -> control.base.tkPtr,
			child -> base.tkPtr,
			child -> row,
			child -> column);
	else if (parent -> control.base.cd == &pymodClassHBox &&
	    ((pymodHBoxDrivers *)pymodClassHBox.drivers) -> addChild)
		(*((pymodHBoxDrivers *)pymodClassHBox.drivers) -> addChild)(
			parent -> control.base.tkPtr,
			child -> base.tkPtr);
	else if (parent -> control.base.cd == &pymodClassVBox &&
	    ((pymodVBoxDrivers *)pymodClassVBox.drivers) -> addChild)
		(*((pymodVBoxDrivers *)pymodClassVBox.drivers) -> addChild)(
			parent -> control.base.tkPtr,
			child -> base.tkPtr);
}
