/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the Control class.
 */


#include <stddef.h>

#include "pymodint.h"

 
/*****************************************************************************/
/* Describe the Control class.                                                */
/*****************************************************************************/
 
/* The properties of this class. */

propDef pymodPropControlParent = {
	"parent",
	instanceProp,
	offsetof(pymodPyControl,control.parent),
	0,
	{0, },
	{0, },
	NULL,
	&pymodClassContainer,
};

propDef pymodPropControlColumn = {
	"column",
	uintProp,
	offsetof(pymodPyControl,control.column),
	0,
	{0, },
	{PROP_VALUE_IS_DEFAULT, 0, NULL, -1},
};

propDef pymodPropControlRow = {
	"row",
	uintProp,
	offsetof(pymodPyControl,control.row),
	0,
	{0, },
	{PROP_VALUE_IS_DEFAULT, 0, NULL, -1},
};

static classPropDef controlProperties[] = {
	{&pymodPropControlParent, },
	{&pymodPropControlColumn, },
	{&pymodPropControlRow, },
	{NULL}
};

/* The class definition. */
 
classDef pymodClassControl = {
        "Control",
	controlProperties,
};


/*
 * Do the first part of initialising a control.
 */

int pymodInitControl1(pymodPyBase *self)
{
	pymodPyControl *cw, *orig;

	/* Add it to the parent container. */

	if (pymodContainerAddChild1(self) < 0)
		return -1;

	/*
	 * If the background color wasn't set, use the nearest parent that had
	 * it set explicitly.  The root of the hierachy is either a Dialog or
	 * an ApplicationWindow and these always have a background set.
	 */

	orig = cw = (pymodPyControl *)self;

	while (cw -> control.background == NULL)
		cw = (pymodPyControl *)(cw -> control.parent);

	if (cw != orig)
	{
		orig -> control.background = cw -> control.background;
		Py_INCREF(cw -> control.base.instance);
	}

	return 0;
}
