/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the Dialog class.
 */


#include <stddef.h>

#include "pymodint.h"


static int dialogCtor(pymodPyBase *,PyObject *,PyObject *);
static PyObject *dialogDone(pymodPyBase *,PyObject *);
static PyObject *dialogHide(pymodPyBase *,PyObject *);
static PyObject *dialogInteract(pymodPyBase *,PyObject *);
static PyObject *dialogShow(pymodPyBase *,PyObject *);
static int dialogSetTitle(pymodPyBase *);


/*****************************************************************************/
/* Describe the Dialog class.                                                */
/*****************************************************************************/

/* The wrapped toolkit object. */

static PyTypeObject dialogType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"__vpDialog__",
	sizeof (pymodPyDialog),
	0,
	pymodDeallocDialog,
};

/* The methods of this class. */

static PyMethodDef dialogMethods[] = {
	{"done", (PyCFunction)dialogDone, METH_VARARGS, },
	{"hide", (PyCFunction)dialogHide, METH_VARARGS, },
	{"interact", (PyCFunction)dialogInteract, METH_VARARGS, },
	{"show", (PyCFunction)dialogShow, METH_VARARGS, },
	{NULL, }
};
 
/* The enums of this class. */

static enumValueDef dialogEnumValuesModality[] = {
	{"Modal", PYMOD_DIALOG_MODALITY_MODAL, },
	{"NonModal", PYMOD_DIALOG_MODALITY_NONMODAL, },
	{NULL}
};

static PyTypeObject dialogEnumTypeModality = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"Modality",
	sizeof (enumObject),
	0,
	0,
	0,
	0,
	0,
	0,
	(reprfunc)pymodEnumRepr,
};

enumDef pymodEnumDialogModality = {
	&dialogEnumTypeModality,
	dialogEnumValuesModality
};

static enumValueDef dialogEnumValuesButton[] = {
	{"Ok", PYMOD_DIALOG_BUTTON_OK, },
	{"Cancel", PYMOD_DIALOG_BUTTON_CANCEL, },
	{"Yes", PYMOD_DIALOG_BUTTON_YES, },
	{"No", PYMOD_DIALOG_BUTTON_NO, },
	{"Continue", PYMOD_DIALOG_BUTTON_CONTINUE, },
	{NULL}
};

static PyTypeObject dialogEnumTypeButton = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"Button",
	sizeof (enumObject),
	0,
	0,
	0,
	0,
	0,
	0,
	(reprfunc)pymodEnumRepr,
};

enumDef pymodEnumDialogButton = {
	&dialogEnumTypeButton,
	dialogEnumValuesButton
};

static enumDef *dialogEnums[] = {
	&pymodEnumDialogButton,
	&pymodEnumDialogModality,
	NULL
};

/* The properties of this class. */

propDef pymodPropDialogButtons = {
	"buttons",
	enumProp,
	offsetof(pymodPyDialog,dialog.buttons),
	PROP_FLAG_LIST,
	{0, },
	{PROP_VALUE_IS_DEFAULT, PYMOD_DIALOG_BUTTON_OK|PYMOD_DIALOG_BUTTON_CANCEL, },
	&pymodEnumDialogButton,
};

propDef pymodPropDialogCloseBox = {
	"closeBox",
	enumProp,
	offsetof(pymodPyDialog,dialog.closeBox),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	&pymodEnumDialogButton,
};

propDef pymodPropDialogContinueText = {
	"continueText",
	stringProp,
	offsetof(pymodPyDialog,dialog.continueText),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
};

propDef pymodPropDialogDefaultButton = {
	"defaultButton",
	enumProp,
	offsetof(pymodPyDialog,dialog.defaultButton),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	&pymodEnumDialogButton,
};

propDef pymodPropDialogModality = {
	"modality",
	enumProp,
	offsetof(pymodPyDialog,dialog.modality),
	0,
	{0, },
	{PROP_VALUE_IS_DEFAULT, PYMOD_DIALOG_MODALITY_MODAL, },
	&pymodEnumDialogModality,
};

propDef pymodPropDialogNoText = {
	"noText",
	stringProp,
	offsetof(pymodPyDialog,dialog.noText),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
};

propDef pymodPropDialogResult = {
	"result",
	pyobjectProp,
	offsetof(pymodPyDialog,dialog.result),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
};

propDef pymodPropDialogYesText = {
	"yesText",
	stringProp,
	offsetof(pymodPyDialog,dialog.yesText),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
};

static classPropDef dialogProperties[] = {
	{&pymodPropContainerChildren, },
	{&pymodPropControlParent, NULL,
		{PROP_VALID_OVERRIDE|PROP_VALID_NONE_OK, },
		{PROP_VALUE_OVERRIDE|PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, }, },
	{&pymodPropTopLevelTitle, dialogSetTitle, },
	{&pymodPropDialogButtons, },
	{&pymodPropDialogCloseBox, pymodNullHandler, },
	{&pymodPropDialogContinueText, },
	{&pymodPropDialogDefaultButton, },
	{&pymodPropDialogModality, },
	{&pymodPropDialogNoText, },
	{&pymodPropDialogResult, pymodNullHandler, },
	{&pymodPropDialogYesText, },
	{NULL}
};


/* The signals of this class. */

static signalDef dialogSignalClickedCancel = {
	"clickedCancel",
};

static signalDef dialogSignalClickedCloseBox = {
	"clickedCloseBox",
};

static signalDef dialogSignalClickedContinue = {
	"clickedContinue",
};

static signalDef dialogSignalClickedNo = {
	"clickedNo",
};

static signalDef dialogSignalClickedOk = {
	"clickedOk",
};

static signalDef dialogSignalClickedYes = {
	"clickedYes",
};

static signalDef *dialogSignals[] = {
	&dialogSignalClickedCancel,
	&dialogSignalClickedCloseBox,
	&dialogSignalClickedContinue,
	&dialogSignalClickedNo,
	&dialogSignalClickedOk,
	&dialogSignalClickedYes,
	NULL
};


/* The class definition. */
 
classDef pymodClassDialog = {
	"Dialog",
	dialogProperties,
	dialogMethods,
	dialogEnums,
	dialogSignals,
	dialogCtor,
	&dialogType,
	NULL,
};


static void doDone(pymodPyDialog *,PyObject *);
static void doShow(pymodPyDialog *);


/*
 * The class ctor.
 */

static int dialogCtor(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	pymodDataDialog *dd = &((pymodPyDialog *)self) -> dialog;
	int defbutt;

	if (pymodParseArgs(&pymodClassDialog,self,args,kwds,dialogProperties,"c") < 0)
		return -1;

	/* Add to the parent. */

	if (dd -> topLevel.container.control.parent != NULL)
		if (pymodContainerAddChild1(self) < 0)
			return -1;

	/*
	 * Set the default button to the most appropriate of those are are
	 * being used.
	 */

	if ((defbutt = dd -> defaultButton) == 0)
		if (dd -> buttons & PYMOD_DIALOG_BUTTON_OK)
			defbutt = PYMOD_DIALOG_BUTTON_OK;
		else if (dd -> buttons & PYMOD_DIALOG_BUTTON_YES)
			defbutt = PYMOD_DIALOG_BUTTON_YES;
		else if (dd -> buttons & PYMOD_DIALOG_BUTTON_CONTINUE)
			defbutt = PYMOD_DIALOG_BUTTON_CONTINUE;
		else if (dd -> buttons & PYMOD_DIALOG_BUTTON_NO)
			defbutt = PYMOD_DIALOG_BUTTON_NO;
		else
			defbutt = PYMOD_DIALOG_BUTTON_CANCEL;

	/*
	 * Call the driver.
	 *
	 * parent is the parent toolkit object, NULL means there is no parent.
	 * title is the text of the dialog title, NULL means use the default.
	 * continueText is the text of any Continue button, NULL means use the
	 * default.
	 * modality is the modality of the dialog.
	 * noText is the text of any No button, NULL means use the default.
	 * yesText is the text of any Yes button, NULL means use the default.
	 */

	if ((dd -> topLevel.container.control.base.tkPtr = (*((pymodDialogDrivers *)pymodClassDialog.drivers) -> ctor)(
		(PyObject *)self,
		getOptTkPtr((pymodPyBase *)dd -> topLevel.container.control.parent),
		getOptString(dd -> topLevel.title),
		dd -> buttons,
		getOptString(dd -> continueText),
		defbutt,
		dd -> modality,
		getOptString(dd -> noText),
		getOptString(dd -> yesText))) == NULL)
	{
		pymodBadToolkitCtor(pymodClassDialog.name);
		return -1;
	}

	/* Handle the background. */

	if (dd -> topLevel.container.control.background == NULL)
	{
		pymodColorDef cd;
 
		/* Get the background from the driver. */
 
		if ((*((pymodDialogDrivers *)pymodClassDialog.drivers) -> getBackground)(
				dd -> topLevel.container.control.base.tkPtr,
				&cd) < 0)
			return -1;
 
		/* Wrap the application supplied color. */
 
		if ((dd -> topLevel.container.control.background = pymodColorNew(&cd)) == NULL)
			return -1;
	}

	dd -> context = pylibGetContext();

	pymodContainerAddChild2(self);

	return 0;
}


/*
 * The method done.
 */

static PyObject *dialogDone(pymodPyBase *self,PyObject *args)
{
	static classPropDef props[] = {
		{&pymodPropDialogResult, },
		{NULL}
	};

	pymodPyDialog *dw;
	PyObject *result;

	if (pymodParseArgs(&pymodClassDialog,self,args,NULL,props,
				"mdone",
				&dw,
				&result) < 0)
		return NULL;

	/* Call the driver. */

	doDone(dw,result);
	Py_DECREF(result);

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * The method hide.
 */

static PyObject *dialogHide(pymodPyBase *self,PyObject *args)
{
	pymodPyDialog *dw;

	if (pymodParseArgs(&pymodClassDialog,self,args,NULL,NULL,
				"mhide",
				&dw) < 0)
		return NULL;

	/* Call the driver. */

	(*((pymodDialogDrivers *)pymodClassDialog.drivers) -> hide)(
			dw -> dialog.topLevel.container.control.base.tkPtr);

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * The method interact.
 */

static PyObject *dialogInteract(pymodPyBase *self,PyObject *args)
{
	pymodPyDialog *dw;

	if (pymodParseArgs(&pymodClassDialog,self,args,NULL,NULL,
				"minteract",
				&dw) < 0)
		return NULL;

	/* Call the driver. */

	doShow(dw);

	Py_INCREF(dw -> dialog.result);
	return dw -> dialog.result;
}


/*
 * The method show.
 */

static PyObject *dialogShow(pymodPyBase *self,PyObject *args)
{
	pymodPyDialog *dw;

	if (pymodParseArgs(&pymodClassDialog,self,args,NULL,NULL,
				"mshow",
				&dw) < 0)
		return NULL;

	/* Call the driver. */

	doShow(dw);

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * The generic button click signal handler.
 */

void pymodSignalDialogClicked(PyObject *self,int butt)
{
	pymodPyDialog *dw = (pymodPyDialog *)self;
	signalDef *sd;

	pylibRestoreContext(dw -> dialog.context);

	/* Get the signal name corresponding to the button. */

	switch (butt)
	{
	case 0:
		sd = &dialogSignalClickedCloseBox;
		break;

	case PYMOD_DIALOG_BUTTON_YES:
		sd = &dialogSignalClickedYes;
		break;
 
	case PYMOD_DIALOG_BUTTON_NO:
		sd = &dialogSignalClickedNo;
		break;
 
	case PYMOD_DIALOG_BUTTON_CONTINUE:
		sd = &dialogSignalClickedContinue;
		break;
 
	case PYMOD_DIALOG_BUTTON_OK:
		sd = &dialogSignalClickedOk;
		break;
 
	case PYMOD_DIALOG_BUTTON_CANCEL:
		sd = &dialogSignalClickedCancel;
		break;
 
	default:
		sd = NULL;

	}

	if (sd == NULL)
	{
		PyErr_Format(PyExc_SystemError,"Internal error: Toolkit returned a bad DialogButton");
		pylibException();
	}
	else if (pymodHandleSignal((pymodPyBase *)self,sd,"()"))
	{
		/* Carry out the default action. */

		if (butt == 0)
			butt = dw -> dialog.closeBox;

		if (butt == 0)
			doDone(dw,Py_None);
		else
		{
			PyObject *buttobj;

			if ((buttobj = pymodConvertEnum(&pymodEnumDialogButton,butt)) != NULL)
			{
				doDone(dw,buttobj);
				Py_DECREF(buttobj);
			}
			else
				pylibException();
		}
	}

	pylibClearContext();
}


/*
 * Call the driver for done() while saving the result.
 */

static void doDone(pymodPyDialog *dw,PyObject *result)
{
	/* Save the result. */

	Py_DECREF(dw -> dialog.result);
	dw -> dialog.result = result;
	Py_INCREF(dw -> dialog.result);

	/* Call the driver. */

	(*((pymodDialogDrivers *)pymodClassDialog.drivers) -> hide)(
			dw -> dialog.topLevel.container.control.base.tkPtr);
}


/*
 * Call the driver for show() while taking care of the global Python lock.
 */

static void doShow(pymodPyDialog *dw)
{
	/* If this is modal then release the global lock. */

	if (dw -> dialog.modality == PYMOD_DIALOG_MODALITY_MODAL)
		pylibClearContext();

	/* Call the driver. */

	(*((pymodDialogDrivers *)pymodClassDialog.drivers) -> show)(
			dw -> dialog.topLevel.container.control.base.tkPtr);

	/* If this is modal then re-acquire the global lock. */

	if (dw -> dialog.modality == PYMOD_DIALOG_MODALITY_MODAL)
		pylibRestoreContext(dw -> dialog.context);
}


/*
 * Handle the setting of the title property.
 */

static int dialogSetTitle(pymodPyBase *self)
{
	pymodDataTopLevel *tld = &((pymodPyTopLevel *)self) -> topLevel;

	return (*((pymodDialogDrivers *)pymodClassDialog.drivers) -> setTitle)(
			tld -> container.control.base.tkPtr,
			getOptString(tld -> title));
}
