/* GDCHART 0.94b  GDC.C  12 Nov 1998 */


#include <gdfontt.h>
#include <gdfonts.h>
#include <gdfontmb.h>
#include <gdfontl.h>
#include <gdfontg.h>

#include "gdchartint.h"


#define PVRED		0x00FF0000
#define PVGRN		0x0000FF00
#define PVBLU		0x000000FF

#define	red(c)		(((c) & PVRED) >> 16)
#define	green(c)	(((c) & PVGRN) >> 8)
#define	blue(c)		((c) & PVBLU)

#define	dim(c)		((c) / 2)

#define contrast(bg)	(((bg) & 0x800000 ? 0 : PVRED) |		\
			 ((bg) & 0x008000 ? 0 : PVGRN) |		\
			 ((bg) & 0x000080 ? 0 : PVBLU))


struct	GDC_FONT_T  GDC_fontc[GDC_numfonts] = { (gdFontPtr)NULL, 8,  5,
						(gdFontPtr)NULL, 8,  5,
						(gdFontPtr)NULL, 12, 6,
						(gdFontPtr)NULL, 13, 7,
						(gdFontPtr)NULL, 16, 8,
						(gdFontPtr)NULL, 15, 9 };

/* ------------------------------------------------------------------- *\ 
 * convert from enum GDC_font_size to gd fonts
 * for now load them all
 *  #defines and #ifdefs might enable loading only needed fonts
\* ------------------------------------------------------------------- */
void
GDC_load_font_conversions()
{
    GDC_fontc[GDC_pad].f     = gdFontTiny;
    GDC_fontc[GDC_TINY].f    = gdFontTiny;
    GDC_fontc[GDC_SMALL].f   = gdFontSmall;
    GDC_fontc[GDC_MEDBOLD].f = gdFontMediumBold;
    GDC_fontc[GDC_LARGE].f   = gdFontLarge;
    GDC_fontc[GDC_GIANT].f   = gdFontGiant;
}

/* ------------------------------------------------------------------ *\ 
 * count (natural) substrings (new line sep)
\* ------------------------------------------------------------------ */
int GDC_cnt_nl( char	*nstr,
	int	*len )		/* strlen - max seg */
{
    int   c           = 1;
    int   max_seg_len = 0;
    int   tmplen      = 0;

    if( !nstr )
	{
	if( len )
	    *len = 0;
	return 0;
	}

    while( *nstr )
	{
	if( *nstr == '\n' )
	    {
	    ++c;
	    max_seg_len = MAX( tmplen, max_seg_len );
	    tmplen = 0;
	    }
	else
	    ++tmplen;
	++nstr;
	}

    if( len )
	*len = MAX( tmplen, max_seg_len );	/* don't forget last seg */
    return c;
}

/* ------------------------------------------------------------------ *\ 
 * gd out a string with '\n's
\* ------------------------------------------------------------------ */
void
GDC_ImageStringNL( gdImagePtr	    im,
		  struct GDC_FONT_T *f,
		  int		    x,
		  int		    y,
		  char		    *str,
		  int		    clr,
		  GDC_justify_t	    justify )
{
    int	    i;
    int	    len;
    int     max_len;
    short   strs_num = GDC_cnt_nl( str, &max_len );
    char    sub_str[max_len+1];

    len      = -1;
    strs_num = -1;
    i = -1;
    do
	{
	++i;
	++len;
	sub_str[len] = *(str+i);
	if( *(str+i) == '\n' ||
	    *(str+i) == '\0' )
	    {
	    int	xpos;

	    sub_str[len] = '\0';
	    ++strs_num;
	    switch( justify )
	      {
	      case GDC_JUSTIFY_LEFT:	xpos = x;		     break;
	      case GDC_JUSTIFY_RIGHT:	xpos = x+f->w*(max_len-len); break;
	      case GDC_JUSTIFY_CENTER:
	      default:			xpos = x+f->w*(max_len-len)/2;
	      }
	    gdImageString( im,
			   f->f,
			   xpos,
			   y + (f->h-1)*strs_num,
			   sub_str,
			   clr );
	    len = -1;
	    }
	}
    while( *(str+i) );
}

/* ------------------------------------------------------------------------ */
void GDC_destroy_image(void *im)
{
    if( im )
	gdImageDestroy( (gdImagePtr)im );
}


/*
 * Either create a new image or return the previously saved image.
 */

gdImage *GDC_create_or_restore_image(GDC_AttributeValues *av,int width,int height)
{
	gdImage *im;

	if (av -> gdc_hold_img && av -> gdc_image != NULL)
		im = (gdImage *)av -> gdc_image;
	else
		im = gdImageCreate(width,height);

	return im;
}


/*
 * Save an image if requested, otherwise destroy it.
 */

void GDC_save_or_destroy_image(GDC_AttributeValues *av,gdImage *im)
{
	if (av -> gdc_hold_img)
		av -> gdc_image = (void *)im;
	else
		gdImageDestroy(im);
}


/*
 * Allocate a color and return its index.  The first call to this will allocate
 * the background color.
 */

int GDC_alloc_color(GDC_AttributeValues *av,gdImage *im,unsigned long color)
{
	/*
	 * The default color is something that contrasts with the background
	 * color.
	 */

	if (color == GDC_DFLTCOLOR)
		color = contrast(av -> gdc_BGColor);

	return gdImageColorResolve(im,red(color),green(color),blue(color));
}


/*
 * Allocate a shadow version of a color and return its index.
 */

int GDC_alloc_color_shadow(GDC_AttributeValues *av,gdImage *im,unsigned long color)
{
	/*
	 * The default color is something that contrasts with the background
	 * color.
	 */

	if (color == GDC_DFLTCOLOR)
		color = contrast(av -> gdc_BGColor);

	return gdImageColorResolve(im,dim(red(color)),dim(green(color)),dim(blue(color)));
}


/*
 * Draw the title of a chart if there is one.
 */

void GDC_draw_title(GDC_AttributeValues *av,gdImage *im,int width)
{
	if (av -> gdc_title != NULL)
	{
		int tlen;
 
		GDC_cnt_nl(av -> gdc_title,&tlen);

		GDC_ImageStringNL(im,
			&GDC_fontc[av -> gdc_title_size],
			width / 2 - tlen * GDC_fontc[av -> gdc_title_size].w / 2,
			1,
			av -> gdc_title,
			GDC_alloc_color(av,im,av -> gdc_TitleColor),
			GDC_JUSTIFY_CENTER );
	}
}


/*
 * Return the height of any title.
 */

int GDC_get_title_height(GDC_AttributeValues *av)
{
	if (av -> gdc_title == NULL)
		return 0;

	/* Add one pixel above and below the title. */

	return 1 + GDC_fontc[av -> gdc_title_size].h * GDC_cnt_nl(av -> gdc_title,NULL) + 1;
}
