/* GDCHART 0.94b  GDCHART.C  18 APR 1999 */

#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <math.h>
#include <string.h>

#include "gdchartint.h"


// scaled translation onto graph
#define PX( x )	    (int)( xorig + (setno*xdepth_3D) + (x)*xscl )
#define PY( y )	    (int)( yorig - (setno*ydepth_3D) + (y)*yscl )
#define PV( y )	    (int)( vyorig - (setno*ydepth_3D) + (y)*vyscl )

#define SET_RECT( gdp, x1, x2, y1, y2 )	gdp[0].x = gdp[3].x = x1,   \
					gdp[0].y = gdp[1].y = y1,   \
					gdp[1].x = gdp[2].x = x2,   \
					gdp[2].y = gdp[3].y = y2

#define SET_3D_POLY( gdp, x1, x2, y1, y2, xoff, yoff )			    \
				gdp[0].x  = x1,        gdp[0].y = y1,	    \
				gdp[1].x  = x1+(xoff), gdp[1].y = y1-yoff,  \
				gdp[2].x  = x2+(xoff), gdp[2].y = y2-yoff,  \
				gdp[3].x  = x2,        gdp[3].y = y2
/* ------------------------------------------------------------------------- */
/* vals in pixels */
/* ref is front plane */
/* allows for intersecting 3D lines      */
/*  (also used for single 3D lines >:-Q  */
struct YS { int y1; int y2; double slope; int lnclr; int shclr; };
static int qcmpr( const void *a, const void *b )
    { if( ((struct YS*)a)->y2 < ((struct YS*)b)->y2 ) return 1;
      if( ((struct YS*)a)->y2 > ((struct YS*)b)->y2 ) return -1;
      return 0; }

static void
draw_3d_line( gdImagePtr    im,
	      int	    y0,
	      int	    x1,
	      int	    x2,
	      int	    y1[],
	      int	    y2[],
	      int	    xdepth,
	      int	    ydepth,
	      int	    num_sets,
	      int	    clr[],
	      int	    clrshd[] )
{
#define F(x,i)	(int)( (double)((x)-x1)*slope[i]+(double)y1[i] )
    double	depth_slope  = xdepth==0? DBL_MAX: (double)ydepth/(double)xdepth;
    double	slope[num_sets];
    int		lnclr[num_sets],
		shclr[num_sets];
    int		i;
    int		x;
    gdPoint	poly[4];
    struct YS	ypts[num_sets];

    for( i=0; i<num_sets; ++i )
	{
	// lnclr[i] = clr[i];
	// shclr[i] = clrshd[i];
	slope[i] = x2==x1? DBL_MAX: (double)(y2[i]-y1[i])/(double)(x2-x1);
	}

    for( x=x1+1; x<=x2; ++x )
	{
	for( i=0; i<num_sets; ++i )			// load set of points
	    {
	    ypts[i].y1    = F(x-1,i);
	    ypts[i].y2    = F(x,i);
	    ypts[i].lnclr = clr[i];
	    ypts[i].shclr = clrshd[i];
	    ypts[i].slope = slope[i];
	    }						// sorted "lowest" first
	qsort( ypts, num_sets, sizeof(struct YS), qcmpr );
							// put out in that order
	for( i=0; i<num_sets; ++i )
	    {						// top
	    SET_3D_POLY( poly, x-1, x, ypts[i].y1, ypts[i].y2, xdepth, ydepth );
	    gdImageFilledPolygon( im, poly, 4,		// depth_slope ever < 0 ?
				  -ypts[i].slope>depth_slope? ypts[i].shclr: ypts[i].lnclr );
	    if( x == x1+1 )				// edging
		gdImageLine( im,
			     x-1, ypts[i].y2,
			     x-1+xdepth, ypts[i].y2-ydepth,
			     -ypts[i].slope<=depth_slope? ypts[i].shclr: ypts[i].lnclr );
	    }
	}
}

/* ------------------------------------------------------------------------- */
/* vals in pixels */
/* ref is front plane */
static void
draw_3d_area( gdImagePtr	im,
	     int		x1,
	     int		x2,
	     int		y0,	    // drawn from 0
	     int		y1,
	     int		y2,
	     int		xdepth,
	     int		ydepth,
	     int		clr,
	     int		clrshd )
{

    gdPoint     poly[4];
    int		y_intercept = 0;				    // if xdepth || ydepth

    if( xdepth || ydepth )
	{
	double	    line_slope   = x2==x1?    DBL_MAX: (double)-(y2-y1) / (double)(x2-x1);
	double	    depth_slope  = xdepth==0? DBL_MAX: (double)ydepth/(double)xdepth;

	y_intercept = (y1 > y0 && y2 < y0) ||			    // line crosses y0
		       (y1 < y0 && y2 > y0)?
			    (int)((1.0/ABS(line_slope))*(double)(ABS(y1-y0)))+x1:
			    0;					    // never

								    // edging along y0 depth
	gdImageLine( im, x1+xdepth, y0-ydepth, x2+xdepth, y0-ydepth, clrshd );

	SET_3D_POLY( poly, x1, x2, y1, y2, xdepth, ydepth );	    // top
	gdImageFilledPolygon( im, poly, 4, line_slope>depth_slope? clrshd: clr );

	SET_3D_POLY( poly, x1, x2, y0, y0, xdepth, ydepth+1 );	// along y axis
	gdImageFilledPolygon( im, poly, 4, clr );

	SET_3D_POLY( poly, x2, x2, y0, y2, xdepth, ydepth );	    // side
	gdImageFilledPolygon( im, poly, 4, clrshd );

	if( y_intercept )
	    gdImageLine( im, y_intercept,        y0,
			     y_intercept+xdepth, y0-ydepth, clrshd );	// edging
	gdImageLine( im, x1, y0, x1+xdepth, y0-ydepth, clrshd );    // edging
	gdImageLine( im, x2, y0, x2+xdepth, y0-ydepth, clrshd );    // edging

	// SET_3D_POLY( poly, x2, x2, y0, y2, xdepth, ydepth );	// side
	// gdImageFilledPolygon( im, poly, 4, clrshd );

	gdImageLine( im, x1, y1, x1+xdepth, y1-ydepth, clrshd );    // edging
	gdImageLine( im, x2, y2, x2+xdepth, y2-ydepth, clrshd );    // edging
	}

    if( y1 == y2 )						// bar rect
	SET_RECT( poly, x1, x2, y0, y1 );			    // front
    else
	{
	poly[0].x = x1;	poly[0].y = y0;
	poly[1].x = x2;	poly[1].y = y0;
	poly[2].x = x2;	poly[2].y = y2;
	poly[3].x = x1;	poly[3].y = y1;
	}
    gdImageFilledPolygon( im, poly, 4, clr );

    gdImageLine( im, x1, y0, x2, y0, clrshd );		// edging along y0

    if( (xdepth || ydepth) &&				    // front edging only on 3D
	(y1<y0 || y2<y0) )				    // and only above y0
	{
	if( y1 > y0 && y2 < y0 )			    // line crosses from below y0
	    gdImageLine( im, y_intercept, y0, x2, y2, clrshd );
	else
	if( y1 < y0 && y2 > y0 )			    // line crosses from above y0
	    gdImageLine( im, x1, y1, y_intercept, y0, clrshd );
	else						    // completely above
	    gdImageLine( im, x1, y1, x2, y2, clrshd );
	}
}

/* ------------------------------------------------------------------------- */
/* vals in pixels */
/* ref is front plane */
static void
draw_3d_bar( gdImagePtr		im,
	     int		x1,
	     int		x2,
	     int		y0,
	     int		yhigh,
	     int		xdepth,
	     int		ydepth,
	     int		clr,
	     int		clrshd )
{
#define SET_3D_BAR( gdp, x1, x2, y1, y2, xoff, yoff )			    \
				gdp[0].x  = x1,        gdp[0].y = y1,	    \
				gdp[1].x  = x1+(xoff), gdp[1].y = y1-yoff,  \
				gdp[2].x  = x2+(xoff), gdp[2].y = y2-yoff,  \
				gdp[3].x  = x2,        gdp[3].y = y2

    gdPoint     poly[4];
    int		usd = MIN( y0, yhigh );				    // up-side-down bars


    if( xdepth || ydepth )
	{
	if( y0 != yhigh )					    // 0 height?
	    {
	    SET_3D_BAR( poly, x2, x2, y0, yhigh, xdepth, ydepth );  // side
	    gdImageFilledPolygon( im, poly, 4, clrshd );
	    }

	SET_3D_BAR( poly, x1, x2, usd, usd, xdepth, ydepth );	    // top
	gdImageFilledPolygon( im, poly, 4, clr );
	}

    SET_RECT( poly, x1, x2, y0, yhigh );			    // front
    gdImageFilledPolygon( im, poly, 4, clr );

    if( xdepth || ydepth )
	gdImageLine( im, x1, usd, x2, usd, clrshd );
}

/* ------------------------------------------------------------------------- */
struct BS { double y1; double y2; int clr; int shclr; };
static int barcmpr( const void *a, const void *b )
    { if( ((struct BS*)a)->y2 < ((struct BS*)b)->y2 ) return -1;
      if( ((struct BS*)a)->y2 > ((struct BS*)b)->y2 ) return 1;
      return 0; }

/* ------------------------------------------------------------------------- */


/*
 * Create a graph.  Returns a pointer to the buffer on the heap containing the
 * PNG image, or NULL if there was an error.
 *
 * width	is the width of the image to create.
 * height	is the height of the image to create.
 * buffer_size	is a pointer to a variable which is set to the size of the
 *		returned buffer in bytes.
 * type		is the type of the graph.
 * av		is a pointer to the attribute values to be used.
 * nrpoints	is the number of data points in a series (along the x-axis).
 * labels	is an array of x-axis label strings.  If it is NULL then no
 *		x-axis labels are displayed, otherwise there must be one label
 *		for each data point in a series.  A label may itself be NULL if
 *		the data point does not have a label.
 * nrseries	is the number of data series (each series comprises nrpoints
 *		data points).
 * series	is a 2D array of data points arranged first by series.
 * comboPts	is an array of data points for use in GDC_*COMBO* graphs.
 *
 * The following attribute values affect the chart:
 *
 *	gdc_generate_image
 *	gdc_hold_img
 *	gdc_image
 *	gdc_transparent_bg
 *	gdc_BGColor
 *	gdc_title
 *	gdc_title_size
 *	gdc_TitleColor
 *	gdc_label_size
 *	gdc_LabelColor
 *	gdc_LineColor
 *	gdc_3d_angle
 *	gdc_3d_depth
 *
 *	gdccht_stack_type
 *	gdccht_bar_width
 *	gdccht_0Shelf
 *	gdccht_grid
 *	gdccht_GridColor
 *	gdccht_border
 *	gdccht_xaxis
 *	gdccht_yaxis
 *	gdccht_yaxis2
 *	gdccht_xaxisfont_size
 *	gdccht_yaxisfont_size
 *	gdccht_xtitle
 *	gdccht_ytitle
 *	gdccht_ytitle2
 *	gdccht_xtitle_size
 *	gdccht_ytitle_size
 *	gdccht_ylabel_fmt
 *	gdccht_ylabel2_fmt
 *	gdccht_XTitleColor
 *	gdccht_YTitleColor
 *	gdccht_YTitle2Color
 *	gdccht_XLabelColor
 *	gdccht_YLabelColor
 *	gdccht_YLabel2Color
 *	gdccht_annotation
 *	gdccht_annotation_font
 *	gdccht_xlabel_density
 *	gdccht_ylabel_density
 *	gdccht_requested_ymin
 *	gdccht_requested_ymax
 *	gdccht_requested_yinterval
 *	gdccht_HLC_style
 *	gdccht_HLC_cap_width
 *	gdccht_num_scatter_pts
 *	gdccht_scatter
 */

void *GDC_out_graph(int	width,int height,int *buffer_size,GDC_CHART_T type,
		    GDC_AttributeValues *av,int nrpoints,char **labels,
		    int nrseries,GDCCHT_DataPoint *series,
		    GDCCHT_DataPoint *comboPts)
{
    void *image_buffer;
    int		i, j, k;

    int		graphwidth;
    int		grapheight;
    gdImagePtr	im;

    double	xorig, yorig, vyorig;
    double	yscl     = 0.0;
    double	vyscl    = 0.0;
    double	xscl     = 0.0;
    double	vhighest = DBL_MIN;
    double	vlowest  = DBL_MAX;
    double	highest  = DBL_MIN;
    double	lowest   = DBL_MAX;
    gdPoint     volpoly[4];

    char	do_vol = (type == GDC_COMBO_HLC_BAR	||
			  type == GDC_COMBO_HLC_AREA	||
			  type == GDC_COMBO_LINE_BAR	||
			  type == GDC_COMBO_LINE_AREA	||
			  type == GDC_3DCOMBO_HLC_BAR	||
			  type == GDC_3DCOMBO_HLC_AREA	||
			  type == GDC_3DCOMBO_LINE_BAR	||
			  type == GDC_3DCOMBO_LINE_AREA);

    char	threeD = (type == GDC_3DAREA		||
			  type == GDC_3DLINE		||
			  type == GDC_3DBAR		||
			  type == GDC_3DHLC		||
			  type == GDC_3DCOMBO_HLC_BAR	||
			  type == GDC_3DCOMBO_HLC_AREA	||
			  type == GDC_3DCOMBO_LINE_BAR	||
			  type == GDC_3DCOMBO_LINE_AREA);

    char	num_hlc_sets =
			 (type == GDC_COMBO_HLC_BAR	||
			  type == GDC_COMBO_HLC_AREA	||
			  type == GDC_3DCOMBO_HLC_BAR	||
			  type == GDC_3DCOMBO_HLC_AREA	||
			  type == GDC_3DHLC		||
			  type == GDC_HLC) ? nrseries : 0;

    char do_bar = (type == GDC_3DBAR ||		// offset X objects to leave
		   type == GDC_BAR );		// room at X(0) and X(n)
						// i.e., not up against Y axes

    char do_ylbl_fractions =			// %f format not given, or
		(!av -> gdccht_ylabel_fmt ||	// format doesn't have a %,g,e,E,f or F
		 strlen(av -> gdccht_ylabel_fmt) == strcspn(av -> gdccht_ylabel_fmt,"%geEfF") );

    double	ylbl_interval  = 0.0;
    int		nrLabelLines   = 0;
    int		xdepth_3Dtotal = 0;
    int		ydepth_3Dtotal = 0;
    int		xdepth_3D      = 0;	// affects PX()
    int		ydepth_3D      = 0;	// affects PY() and PV()
    int		hlf_barwdth    = 0;	// half bar widths
    int		hlf_hlccapwdth = 0;	// half cap widths
    int		annote_len     = 0,
		annote_hgt     = 0;

    /* args */
    int		setno = 0;		// affects PX() and PY()
    int		BGColor,
		LabelColor,
		LineColor,
		GridColor,
		comboColor[nrpoints],
		AnnoteColor,
		ptsColor[nrseries][nrpoints];
					// shade colors only with 3D
    int		ptsColorShd[nrseries][nrpoints];

	GDCCHT_DataPoint *pts[nrseries][nrpoints];

	/* Arrange the points in a table. */

	for (i = 0; i < nrseries; ++i)
		for (j = 0; j < nrpoints; ++j)
			pts[i][j] = series++;

	image_buffer = NULL;
	*buffer_size = 0;

    /* idiot checks */
    if( width<=0 || height<=0 || nrpoints <= 0)
	return NULL;

    GDC_load_font_conversions();

	if (av -> gdccht_xaxisfont_size == GDC_DEFAULT_FONT)
		av -> gdccht_xaxisfont_size = av -> gdc_label_size;

	if (av -> gdccht_yaxisfont_size == GDC_DEFAULT_FONT)
		av -> gdccht_yaxisfont_size = av -> gdc_label_size;

    /* ----- highest & lowest values ----- */
    if( av -> gdccht_stack_type == GDC_STACK_LAYERED )	// need to walk sideways
	for( j=0; j<nrpoints; ++j )
	    {
	    double   neg_set_sum = 0.0,
		    pos_set_sum = 0.0;
	    for( i=0; i<nrseries; ++i )
		if( pts[i][j] -> simple != DBL_MIN )
		    if( pts[i][j] -> simple < 0.0 )
			neg_set_sum += pts[i][j] -> simple;
		    else
			pos_set_sum += pts[i][j] -> simple;
	    lowest  = MIN( lowest,  MIN(neg_set_sum,pos_set_sum) );
	    highest = MAX( highest, MAX(neg_set_sum,pos_set_sum) );
	    }
    else
		for (i = 0; i < nrseries; ++i)
			for (j = 0; j < nrpoints; ++j)
			{
				double high, low;

				if (num_hlc_sets == 0)
					high = low = pts[i][j] -> simple;
				else
				{
					high = pts[i][j] -> high;
					low = pts[i][j] -> low;
				}

				if (high != DBL_MIN)
					highest = MAX(high,highest);

				if (low != DBL_MIN)
					lowest = MAX(low,lowest);
			}

    if( av -> gdccht_scatter )
      for( i=0; i<av -> gdccht_num_scatter_pts; ++i )
	{
	highest = MAX( (av -> gdccht_scatter+i)->val, highest );
	lowest  = MIN( (av -> gdccht_scatter+i)->val, lowest  );
	}

	if (do_vol)			// for now only one combo set allowed
	{
		for (j = 0; j < nrpoints; ++j)
		{
			double val;

			if ((val = comboPts[j].simple) != DBL_MIN )
			{
				vhighest = MAX(val,vhighest);
				vlowest = MIN(val,vlowest);
			}
		}

		if (vhighest == DBL_MIN)	// no values
			vhighest = 1.0;		// for scaling, need a range
		else if (vhighest < 0.0)
			vhighest = 0.0;

		if (vlowest > 0.0)
			vlowest = 0.0;		// vol should always start at 0
	}

    if( lowest == DBL_MAX )
	lowest = 0.0;
    if( highest == DBL_MIN )
	highest = 1.0;				// need a range
    if( type == GDC_AREA  ||			// bars and area should always start at 0
	type == GDC_BAR   ||
	type == GDC_3DBAR ||
	type == GDC_3DAREA )
	if( highest < 0.0 )
	    highest = 0.0;
	else
	if( lowest > 0.0 )			// negs should be drawn from 0
	    lowest = 0.0;

    if( av -> gdccht_requested_ymin != DBL_MIN && av -> gdccht_requested_ymin < lowest )
	lowest = av -> gdccht_requested_ymin;
    if( av -> gdccht_requested_ymax != DBL_MIN && av -> gdccht_requested_ymax > highest )
	highest = av -> gdccht_requested_ymax;
    

    /* ----- graph height and width within the gif height width ----- */
    /* grapheight/height is the actual size of the scalable graph */
    {
    int	title_hgt = GDC_get_title_height(av);
    int	xlabel_hgt = 0;
    int	xtitle_hgt = av -> gdccht_xtitle? 1+GDC_fontc[av -> gdccht_xtitle_size].h+1: 0;
    int	ytitle_hgt = av -> gdccht_ytitle? 1+GDC_fontc[av -> gdccht_ytitle_size].h+1: 0;
    int	vtitle_hgt = do_vol&&av -> gdccht_ytitle2? 1+GDC_fontc[av -> gdccht_ytitle_size].h+1: 0;
    int	ylabel_wth = 0;
    int	vlabel_wth = 0;

    int	xtics       = av -> gdccht_grid? 1+2: 0;
    int	ytics       = av -> gdccht_grid? 1+3: 0;
    int	vtics       = av -> gdccht_grid&&do_vol? 3+1: 0;
	int height_below;


#define	HYP_DEPTH   ( (double)((width+height)/2) * ((double)av -> gdc_3d_depth)/100.0 )
#define RAD_DEPTH   ( (double)av -> gdc_3d_angle*2*M_PI/360 )
    xdepth_3D      = threeD? (int)( cos(RAD_DEPTH) * HYP_DEPTH ): 0;
    ydepth_3D      = threeD? (int)( sin(RAD_DEPTH) * HYP_DEPTH ): 0;
    xdepth_3Dtotal = xdepth_3D*(av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets:
									       nrseries:
								 1 );
    ydepth_3Dtotal = ydepth_3D*(av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets:
									       nrseries:
								 1 );
    annote_hgt = av -> gdccht_annotation && *(av -> gdccht_annotation->note)?
		    1 +						/* space to note */
		    (1+GDC_fontc[av -> gdccht_annotation_font].h) *	/* number of '\n' substrs */
		    GDC_cnt_nl(av -> gdccht_annotation->note,&annote_len) +
		    1 +						/* space under note */
		    2: 0;					/* space to chart */
    annote_len *= GDC_fontc[av -> gdccht_annotation_font].w;

		if (av -> gdccht_xaxis && labels)
		{
			int biggest = 0;

			for (i = 0; i < nrpoints; ++i)
			{
				int nrlines, len;

				nrlines = GDC_cnt_nl(labels[i],&len);

				if (len > biggest)
					biggest = len;

				if (nrlines > nrLabelLines)
					nrLabelLines = nrlines;
			}

			xlabel_hgt = 1 + biggest*GDC_fontc[av -> gdccht_xaxisfont_size].w + 1;

			height_below = xtics + xtitle_hgt + xlabel_hgt;
		}
		else
			height_below = 0;

		/*
		 * Make sure that there is enough room for any y-axis label
		 * that is below the x-axis.
		 */

		if (av -> gdccht_yaxis && (GDC_fontc[av -> gdccht_yaxisfont_size].h + 1) / 2 > height_below)
			height_below = (GDC_fontc[av -> gdccht_yaxisfont_size].h + 1) / 2;
    
    grapheight = height - ( height_below +
			       title_hgt      +
			       annote_hgt     +
			       ydepth_3Dtotal +
			       2 );

    /* ----- y labels intervals ----- */
    {
    double   tmp_highest;
							    /* possible y gridline points */
    double   ypoints[] = { 1.0/64.0, 1.0/32.0, 1.0/16.0, 1.0/8.0, 1.0/4.0, 1.0/2.0,
			  1.0,      2.0,      3.0,      5.0,     10.0,    25.0,
			  50.0,     100.0,    250.0,    500.0,   1000.0,  2500,    5000.0,
			  10000.0,  25000.0,  50000.0,  100000.0,500000.0,1000000, 5000000,
			  10000000 };
    #define NUM_YPOINTS	(sizeof(ypoints) / sizeof(double))
    int	    max_num_ylbls;
    int	    longest_ylblen = 0;

	/* maximum y lables that'll fit... */

    max_num_ylbls = grapheight / (3+GDC_fontc[av -> gdccht_yaxisfont_size==GDC_TINY? av -> gdccht_yaxisfont_size+1:
				av -> gdccht_yaxisfont_size].h);

    /* See if there is sufficient height. */

    if( max_num_ylbls < 3 )
	return NULL;

	/* one "space" interval above + below */

    for( i=1; i<NUM_YPOINTS; ++i )
	if( (highest-lowest)/ypoints[i] < ((double)max_num_ylbls-(1.0+1.0))
					    * (double)av -> gdccht_ylabel_density/100.0 )
	    break;
    /* gotta go through the above loop to catch the 'tweeners :-| */

    ylbl_interval = av -> gdccht_requested_yinterval != DBL_MIN &&
		    av -> gdccht_requested_yinterval > ypoints[i-1]?   av -> gdccht_requested_yinterval:
							      ypoints[i-1];

							/* perform floating point remainders */
							/* gonculate largest interval-point < lowest */
    if( lowest != 0.0 &&
	lowest != av -> gdccht_requested_ymin )
	{
	if( lowest < 0.0 )
	    lowest -= ylbl_interval;
	// lowest = (lowest-ypoints[0]) -
	//	    ( ( ((lowest-ypoints[0])/ylbl_interval)*ylbl_interval ) -
	//	       ( (double)((int)((lowest-ypoints[0])/ylbl_interval))*ylbl_interval ) );
	lowest = ylbl_interval * (double)(int)((lowest-ypoints[0])/ylbl_interval);
	}
							/* find smallest interval-point > highest */
    tmp_highest = lowest;
    do	// while( (tmp_highest += ylbl_interval) <= highest )
	{
	int	nmrtr, dmntr, whole;
	int	lbl_len;
	char	foo[100];

	if( av -> gdccht_yaxis )
	    {						/* XPG2 compatibility */
	    sprintf( foo, do_ylbl_fractions? "%.0f": av -> gdccht_ylabel_fmt, tmp_highest );
	    lbl_len = ylbl_interval<1.0? strlen( GDC_price_to_str(tmp_highest,
							      &nmrtr,
							      &dmntr,
							      &whole,
							      do_ylbl_fractions? NULL: av -> gdccht_ylabel_fmt) ):
					 strlen( foo );
	    longest_ylblen = MAX( longest_ylblen, lbl_len );
	    }
	} while( (tmp_highest += ylbl_interval) <= highest );
    ylabel_wth = longest_ylblen * GDC_fontc[av -> gdccht_yaxisfont_size].w;
    highest = av -> gdccht_requested_ymax==DBL_MIN? tmp_highest:
					       MAX( av -> gdccht_requested_ymax, highest );

    if( do_vol )
	{
	double	num_yintrvls = (highest-lowest) / ylbl_interval;
							    /* no skyscrapers */
	if( vhighest != 0.0 )
	    vhighest += (vhighest-vlowest) / (num_yintrvls*2.0);
	if( vlowest != 0.0 )
	    vlowest -= (vhighest-vlowest) / (num_yintrvls*2.0);

	if( av -> gdccht_yaxis2 )
	    {
	    char    svlongest[100];
	    int	    lbl_len_low  = sprintf( svlongest, av -> gdccht_ylabel2_fmt? av -> gdccht_ylabel2_fmt: "%.0f", vlowest );
	    int	    lbl_len_high = sprintf( svlongest, av -> gdccht_ylabel2_fmt? av -> gdccht_ylabel2_fmt: "%.0f", vhighest );
	    vlabel_wth = 1
			 + MAX( lbl_len_low,lbl_len_high ) * GDC_fontc[av -> gdccht_yaxisfont_size].w;
	    }
	}
    }

    graphwidth = width - (ytitle_hgt +
			  ylabel_wth +
			  ytics +
			  vtics +
			  vtitle_hgt +
			  vlabel_wth +
			  xdepth_3Dtotal);

    /* ----- scale to gif size ----- */
    /* offset to 0 at lower left (where it should be) */
    xscl = (double)(graphwidth-xdepth_3Dtotal) / (double)(nrpoints + (do_bar?2:0));
    yscl = -((double)grapheight) / (double)(highest-lowest);
    if( do_vol )
	{
	double	hilow_diff = vhighest-vlowest==0.0? 1.0: vhighest-vlowest;

	vyscl = -((double)grapheight) / hilow_diff;
	vyorig = (double)grapheight
		 + ABS(vyscl) * MIN(vlowest,vhighest)
		 + ydepth_3Dtotal
		 + title_hgt
		 + annote_hgt;
	}
    xorig = (double)( width - ( graphwidth +
				  vtitle_hgt +
				  vtics      +
				  vlabel_wth ) );

//  yorig = (double)grapheight + ABS(yscl * lowest) + ydepth_3Dtotal + title_hgt;
    yorig = (double)grapheight
		+ ABS(yscl) * MIN(lowest,highest)
		+ ydepth_3Dtotal
		+ title_hgt
		+ annote_hgt;

    hlf_barwdth     = (int)( (double)(PX(2)-PX(1)) * (((double)av -> gdccht_bar_width/100.0)/2.0) );   // used only for bars
    hlf_hlccapwdth  = (int)( (double)(PX(2)-PX(1)) * (((double)av -> gdccht_HLC_cap_width/100.0)/2.0) );
    }
    // scaled, sized, ready


    /* ----- OK start the graphic ----- */
    im = GDC_create_or_restore_image(av,width,height);


    BGColor        = GDC_alloc_color(av,im,av -> gdc_BGColor);
    LabelColor     = GDC_alloc_color(av,im,av -> gdc_LabelColor);
    LineColor      = GDC_alloc_color(av,im,av -> gdc_LineColor);
    GridColor      = GDC_alloc_color(av,im,av -> gdccht_GridColor);

	if (do_vol)
		for (i = 0; i < nrpoints; ++i)
			comboColor[i] = GDC_alloc_color(av,im,comboPts[i].color);

    if( av -> gdccht_annotation )
	AnnoteColor = GDC_alloc_color(av,im,av -> gdccht_annotation->color);

	for (i = 0; i < nrseries; ++i)
		for (j = 0; j < nrpoints; ++j)
		{
			unsigned long col = pts[i][j] -> color;

			ptsColor[i][j] = GDC_alloc_color(av,im,col);

			if (threeD)
				ptsColorShd[i][j] = GDC_alloc_color_shadow(av,im,col);
		}

    if( av -> gdc_transparent_bg )
	gdImageColorTransparent( im, BGColor );

	GDC_draw_title(av,im,width);

    if( av -> gdccht_xtitle )
	{
	int titlecolor = GDC_alloc_color(av,im,av -> gdccht_XTitleColor);

	gdImageString( im,
		       GDC_fontc[av -> gdccht_xtitle_size].f,
		       width/2 - strlen(av -> gdccht_xtitle)*GDC_fontc[av -> gdccht_xtitle_size].w/2,
		       height-GDC_fontc[av -> gdccht_xtitle_size].h-1,
		       av -> gdccht_xtitle,
		       titlecolor );
	}


    /* ----- start drawing ----- */
    /* ----- backmost first - grid & labels ----- */
    if( av -> gdccht_grid || av -> gdccht_yaxis )
	{   /* grid lines & y label(s) */
	double	tmp_y = lowest;
	int	labelcolor = av -> gdccht_YLabelColor==GDC_DFLTCOLOR? 
			     LabelColor: GDC_alloc_color(av,im,av -> gdccht_YLabelColor);
	int	label2color = av -> gdccht_YLabel2Color==GDC_DFLTCOLOR? 
			      LabelColor: GDC_alloc_color(av,im,av -> gdccht_YLabel2Color);

	/* step from lowest to highest puting in labels and grid at interval points */
	/* since now "odd" intervals may be requested, try to step starting at 0,   */
	/* if lowest < 0 < highest                                                  */
	for( i=-1; i<=1; i+=2 )					// -1, 1
	    {
	    if( i == -1 )   if( lowest >= 0.0 )			//  all pos plotting
				continue;
			    else
				tmp_y = MIN( 0, highest );	//  step down to lowest

	    if( i == 1 )    if( highest <= 0.0 )		//  all neg plotting
				continue;
			    else
				tmp_y = MAX( 0, lowest );	//  step up to highest


//	    if( !(highest > 0 && lowest < 0) )			// doesn't straddle 0
//		{
//		if( i == -1 )					// only do once: normal
//		    continue;
//		}
//	    else
//		tmp_y = 0;

	    do	// while( (tmp_y (+-)= ylbl_interval) < [highest,lowest] )
		{
		int	n, d, w;
		char	nmrtr[3+1], dmntr[3+1], whole[8];
		char	all_whole = ylbl_interval<1.0? FALSE: TRUE;

		char	*ylbl_str = GDC_price_to_str( tmp_y,&n,&d,&w,
						  do_ylbl_fractions? NULL: av -> gdccht_ylabel_fmt );
		if( do_ylbl_fractions )
		    {
		    sprintf( nmrtr, "%d", n );
		    sprintf( dmntr, "%d", d );
		    sprintf( whole, "%d", w );
		    }

		if( av -> gdccht_grid )
		    {
		    int	x1, x2, y1, y2;
		    // int  gridline_clr = tmp_y == 0.0? LineColor: GridColor;
									// tics
		    x1 = PX(0);	    y1 = PY(tmp_y);
		    gdImageLine( im, x1-2, y1, x1, y1, GridColor );
		    setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets:
									   nrseries:
							     1;		// backmost
		    x2 = PX(0);	    y2 = PY(tmp_y);			// w/ new setno
		    gdImageLine( im, x1, y1, x2, y2, GridColor );	// depth for 3Ds
		    gdImageLine( im, x2, y2, PX(nrpoints-1+(do_bar?2:0)), y2, GridColor );
		    setno = 0;						// set back to foremost
		    }
		if( av -> gdccht_yaxis )
		    if( do_ylbl_fractions )
			{
			if( w || (!w && !n && !d) )
			    {
			    gdImageString( im,
					   GDC_fontc[av -> gdccht_yaxisfont_size].f,
					   PX(0)-2-strlen(whole)*GDC_fontc[av -> gdccht_yaxisfont_size].w
						  - ( (!all_whole)?
							(strlen(nmrtr)*GDC_fontc[av -> gdccht_yaxisfont_size-1].w +
							 GDC_fontc[av -> gdccht_yaxisfont_size].w                 +
							 strlen(nmrtr)*GDC_fontc[av -> gdccht_yaxisfont_size-1].w) :
							1 ),
					   PY(tmp_y)-GDC_fontc[av -> gdccht_yaxisfont_size].h/2,
					   whole,
					   labelcolor );
			    }
			if( n )
			    {
			    gdImageString( im,
					   GDC_fontc[av -> gdccht_yaxisfont_size-1].f,
					   PX(0)-2-strlen(nmrtr)*GDC_fontc[av -> gdccht_yaxisfont_size-1].w
						  -GDC_fontc[av -> gdccht_yaxisfont_size].w
						  -strlen(nmrtr)*GDC_fontc[av -> gdccht_yaxisfont_size-1].w + 1,
					   PY(tmp_y)-GDC_fontc[av -> gdccht_yaxisfont_size].h/2 + 1,
					   nmrtr,
					   labelcolor );
			    gdImageString( im,
					   GDC_fontc[av -> gdccht_yaxisfont_size].f,
					   PX(0)-2-GDC_fontc[av -> gdccht_yaxisfont_size].w
						  -strlen(nmrtr)*GDC_fontc[av -> gdccht_yaxisfont_size-1].w,
					   PY(tmp_y)-GDC_fontc[av -> gdccht_yaxisfont_size].h/2,
					   "/",
					   labelcolor );
			    gdImageString( im,
					   GDC_fontc[av -> gdccht_yaxisfont_size-1].f,
					   PX(0)-2-strlen(nmrtr)*GDC_fontc[av -> gdccht_yaxisfont_size-1].w - 2,
					   PY(tmp_y)-GDC_fontc[av -> gdccht_yaxisfont_size].h/2 + 3,
					   dmntr,
					   labelcolor );
			    }
			}
		    else
			gdImageString( im,
				       GDC_fontc[av -> gdccht_yaxisfont_size].f,
				       PX(0)-2-strlen(ylbl_str)*GDC_fontc[av -> gdccht_yaxisfont_size].w,
				       PY(tmp_y)-GDC_fontc[av -> gdccht_yaxisfont_size].h/2,
				       ylbl_str,
				       labelcolor );


		if( do_vol && av -> gdccht_yaxis2 )
		    {
		    char    vylbl[16];
										/* opposite of PV(y) */
		    sprintf( vylbl,
			     av -> gdccht_ylabel2_fmt? av -> gdccht_ylabel2_fmt: "%.0f",
			     ((double)(PY(tmp_y)+(setno*ydepth_3D)-vyorig))/vyscl );

		    setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets:
									   nrseries:
							     1; // backmost
		    gdImageLine( im, PX(nrpoints-1+(do_bar?2:0)), PY(tmp_y),
				     PX(nrpoints-1+(do_bar?2:0))+3, PY(tmp_y), GridColor );
		    if( atof(vylbl) == 0.0 )					/* rounding can cause -0 */
			strcpy( vylbl, "0" );
		    gdImageString( im,
				   GDC_fontc[av -> gdccht_yaxisfont_size].f,
				   PX(nrpoints-1+(do_bar?2:0))+6,
				   PY(tmp_y)-GDC_fontc[av -> gdccht_yaxisfont_size].h/2,
				   vylbl,
				   label2color );
		    setno = 0;
		    }
		}
	    while( ((i>0) && ((tmp_y += ylbl_interval) < highest)) ||
		   ((i<0) && ((tmp_y -= ylbl_interval) > lowest)) );
	    }

	/* catch last (bottom) grid line - specific to an "off" requested interval */
	if( av -> gdccht_grid && threeD )
	    {
	    setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets:
								   nrseries:
						     1;		// backmost
	    gdImageLine( im, PX(0), PY(lowest), PX(nrpoints-1+(do_bar?2:0)), PY(lowest), GridColor );
	    setno = 0;						// set back to foremost
	    }

	/* vy axis title */
	if( do_vol && av -> gdccht_ytitle2 )
	    {
	    int	ytit_len = strlen(av -> gdccht_ytitle2)*GDC_fontc[av -> gdccht_ytitle_size].w;
	    int	titlecolor = GDC_alloc_color(av,im,av -> gdccht_YTitle2Color);

	    gdImageStringUp( im,
			     GDC_fontc[av -> gdccht_ytitle_size].f,
			     width-(1+GDC_fontc[av -> gdccht_ytitle_size].h),
			     height/2 + ytit_len/2,
			     av -> gdccht_ytitle2,
			     titlecolor );
	    }

	/* y axis title */
	if( av -> gdccht_ytitle )
	    {
	    int	ytit_len = strlen(av -> gdccht_ytitle)*GDC_fontc[av -> gdccht_ytitle_size].w;
	    int	titlecolor = GDC_alloc_color(av,im,av -> gdccht_YTitleColor);

	    gdImageStringUp( im,
			     GDC_fontc[av -> gdccht_ytitle_size].f,
			     0,
			     height/2 + ytit_len/2,
			     av -> gdccht_ytitle,
			     titlecolor );
	    }
	}

    /* interviening set grids */
    /*  0 < setno < nrseries   non-inclusive, they've already been covered */
    if( av -> gdccht_grid && threeD )
	{
	for( setno=(av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets: nrseries: 1) - 1;
	     setno > 0;
	     --setno )
	    {
	    gdImageLine( im, PX(0), PY(lowest), PX(0), PY(highest), GridColor );
	    gdImageLine( im, PX(0), PY(lowest), PX(nrpoints-1+(do_bar?2:0)), PY(lowest), GridColor );
	    }
	setno = 0;
	}

    if( ( av -> gdccht_grid || av -> gdccht_0Shelf ) &&				/* line color grid at 0 */
	  (lowest < 0.0 && highest > 0.0) )
	{
	int x1, x2, y1, y2;
								// tics
	x1 = PX(0);	y1 = PY(0);
	gdImageLine( im, x1-2, y1, x1, y1, LineColor );
	setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets:
							       nrseries:
						 1;		// backmost
	x2 = PX(0);	y2 = PY(0);				// w/ new setno
	gdImageLine( im, x1, y1, x2, y2, LineColor );		// depth for 3Ds
	gdImageLine( im, x2, y2, PX(nrpoints-1+(do_bar?2:0)), y2, LineColor );
	setno = 0;						// set back to foremost
	}


    /* x ticks and xlables */
    if( av -> gdccht_grid || av -> gdccht_xaxis )
	{
	int num_labels;
	int	labelcolor = av -> gdccht_XLabelColor==GDC_DFLTCOLOR? 
			     LabelColor: GDC_alloc_color(av,im,av -> gdccht_XLabelColor);

	if (av -> gdccht_xlabel_density < 0)
		num_labels = nrpoints;
	else
		num_labels = (graphwidth * av -> gdccht_xlabel_density) /
				(GDC_fontc[av -> gdccht_xaxisfont_size].h * nrLabelLines * 100);

	for( i=0; i<nrpoints+(do_bar?2:0); ++i )
	    if( (i%(1+nrpoints/num_labels) == 0) ||			// # x labels are regulated
		num_labels >= nrpoints)
		{
		int xi = do_bar? i-1: i;

		if( av -> gdccht_grid )
		    {
		    int	x1, x2, y1, y2;
									// tics
		    x1 = PX(i);	    y1 = PY(lowest);
		    gdImageLine( im, x1, y1, x1,  y1+2, GridColor );
		    setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets:
									   nrseries:
							     1; // backmost
		    x2 = PX(i);	    y2 = PY(lowest);
		    gdImageLine( im, x1, y1, x2,  y2, GridColor );	// depth perspective
		    gdImageLine( im, x2, y2, x2,  PY(highest), GridColor );
		    setno = 0;						// reset to foremost
		    }

		if( !do_bar || (i>0 && xi<nrpoints) )
		    if( av -> gdccht_xaxis && labels && labels[xi] && *(labels[xi]) )
			{
			/* waiting for GDC_ImageStringUpNL() */
#define			LBX	GDC_fontc[av -> gdccht_xaxisfont_size]
			int	xlen = 0;
			short	xstrs_num = GDC_cnt_nl( labels[xi], &xlen );
			char	sub_label[xlen+1];
//			int	labels_strt = -1+ PX((double)i+(double)(do_bar?((double)nrpoints/(double)num_labels):0.0)) - (int)((double)(LBX.h-2)*((double)xstrs_num/2.0));
			int	label_strt = -1+ PX(i) - (int)((double)(LBX.h-2)*((double)xstrs_num/2.0));

			xlen      = -1;
			xstrs_num = -1;
			j = -1;
			do
			    {
			    ++j;
			    ++xlen;
			    sub_label[xlen] = labels[xi][j];
			    if( labels[xi][j] == '\n' ||
				labels[xi][j] == '\0' )
				{
				sub_label[xlen] = '\0';
				++xstrs_num;
				gdImageStringUp( im,
						 LBX.f,
						 label_strt + (LBX.h-1)*xstrs_num,
						 PY(lowest) + 2 + 1 + LBX.w*xlen,
						 sub_label,
						 labelcolor );
				xlen = -1;
				}
			    } while( labels[xi][j] );
#undef LBX
			}
		}
	}

    /* ----- solid poly region (volume) ----- */
    /*  so that grid lines appear under solid */
    if( do_vol )
	{
	setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets:
							       nrseries:
						 1; // backmost
	if( type == GDC_COMBO_HLC_BAR    ||
	    type == GDC_COMBO_LINE_BAR   ||
	    type == GDC_3DCOMBO_LINE_BAR ||
	    type == GDC_3DCOMBO_HLC_BAR )
	    {
	    if( comboPts[0].simple != DBL_MIN )
		draw_3d_bar( im, PX(0), PX(0)+hlf_barwdth,
				 PV(0), PV(comboPts[0].simple),
				 0, 0,
				 comboColor[0],
				 comboColor[0] );
	    for( i=1; i<nrpoints-1; ++i )
		if( comboPts[i].simple != DBL_MIN )
		    draw_3d_bar( im, PX(i)-hlf_barwdth, PX(i)+hlf_barwdth,
				     PV(0), PV(comboPts[i].simple),
				     0, 0,
				     comboColor[i],
				     comboColor[i] );
	    if( comboPts[i].simple != DBL_MIN )
		draw_3d_bar( im, PX(i)-hlf_barwdth, PX(i),
				 PV(0), PV(comboPts[i].simple),
				 0, 0,
				 comboColor[i],
				 comboColor[i] );
	    }
	else
	if( type == GDC_COMBO_HLC_AREA   ||
	    type == GDC_COMBO_LINE_AREA  ||
	    type == GDC_3DCOMBO_LINE_AREA||
	    type == GDC_3DCOMBO_HLC_AREA )
	    for( i=1; i<nrpoints; ++i )
		if( comboPts[i-1].simple != DBL_MIN && comboPts[i].simple != DBL_MIN )
		    draw_3d_area( im, PX(i-1), PX(i),
				     PV(0), PV(comboPts[i-1].simple), PV(comboPts[i].simple),
				     0, 0,
				     comboColor[i],
				     comboColor[i] );
	setno = 0;
	}	// volume polys done

    if( av -> gdccht_annotation && threeD )	/* back half of annotation line */
	{
	int x1 = PX(av -> gdccht_annotation->point+(do_bar?1:0)),
	    y1 = PY(lowest);
	setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets:
							       nrseries:
						 1; // backmost
	gdImageLine( im, x1, y1, PX(av -> gdccht_annotation->point+(do_bar?1:0)), PY(lowest), AnnoteColor );
	gdImageLine( im, PX(av -> gdccht_annotation->point+(do_bar?1:0)), PY(lowest),
			 PX(av -> gdccht_annotation->point+(do_bar?1:0)), PY(highest)-2, AnnoteColor );
	setno = 0;
	}

    /* ---------- start plotting the data ---------- */
    switch( type )
	{
	case GDC_3DBAR:			/* depth, width, y interval need to allow for whitespace between bars */
	case GDC_BAR:
	/* --------- */
	switch( av -> gdccht_stack_type )
	    {
	    case GDC_STACK_BEHIND:
	    for( setno=nrseries-1; setno>=0; --setno )	    // back sets first   PX, PY depth
		for( i=0; i<nrpoints; ++i )
		    if( pts[setno][i] -> simple != DBL_MIN )
			draw_3d_bar( im, PX(i+(do_bar?1:0))-hlf_barwdth, PX(i+(do_bar?1:0))+hlf_barwdth,
					 PY(0), PY(pts[setno][i] -> simple),
					 xdepth_3D, ydepth_3D,
					 ptsColor[setno][i],
					 threeD? ptsColorShd[setno][i]: ptsColor[setno][i] );
	    setno = 0;
	    break;

	    case GDC_STACK_LAYERED:
		{
		double	lasty[nrpoints];
		j = 0;
		for( i=0; i<nrpoints; ++i )
		    {
		    struct BS	barset[nrseries];
		    double	lasty_pos = 0.0;
		    double	lasty_neg = 0.0;
		    int		k;

		    for( j=0, k=0; j<nrseries; ++j )
			{
			if( pts[j][i] -> simple != DBL_MIN )
			    {
			    if( pts[j][i] -> simple < 0.0 )
				{
				barset[k].y1 = lasty_neg;
				barset[k].y2 = pts[j][i] -> simple + lasty_neg;
				lasty_neg    = barset[k].y2;
				}
			    else
				{
				barset[k].y1 = lasty_pos;
				barset[k].y2 = pts[j][i] -> simple + lasty_pos;
				lasty_pos    = barset[k].y2;
				}
			    barset[k].clr   = ptsColor[j][i];
			    barset[k].shclr = threeD? ptsColorShd[j][i]: ptsColor[j][i];
			    ++k;
			    }
			}
		    qsort( barset, k, sizeof(struct BS), barcmpr );

		    for( j=0; j<k; ++j )
			{
			draw_3d_bar( im,
				     PX(i+(do_bar?1:0))-hlf_barwdth, PX(i+(do_bar?1:0))+hlf_barwdth,
				     PY(barset[j].y1), PY(barset[j].y2),
				     xdepth_3D, ydepth_3D,
				     barset[j].clr,
				     barset[j].shclr );
			}
		    }
		}
		break;

	    case GDC_STACK_BESIDE:
		{						// h/.5, h/1, h/1.5, h/2, ...
		int new_barwdth = (int)( (double)hlf_barwdth / ((double)nrseries/2.0) );
		for( i=0; i<nrpoints; ++i )
		    for( j=0; j<nrseries; ++j )
			if( pts[j][i] -> simple != DBL_MIN )
			    draw_3d_bar( im, PX(i+(do_bar?1:0))-hlf_barwdth+new_barwdth*j+1,
					     PX(i+(do_bar?1:0))-hlf_barwdth+new_barwdth*(j+1),
					     PY(0), PY(pts[j][i] -> simple),
					     xdepth_3D, ydepth_3D,
					     ptsColor[j][i],
					     threeD? ptsColorShd[j][i]: ptsColor[j][i] );
		    }
		break;
	    }
	    break;

	case GDC_LINE:
	case GDC_COMBO_LINE_BAR:
	case GDC_COMBO_LINE_AREA:
	    for( j=nrseries-1; j>=0; --j )
		for( i=1; i<nrpoints; ++i )
		    if( pts[j][i-1] -> simple != DBL_MIN && pts[j][i] -> simple != DBL_MIN )
			{
			gdImageLine( im, PX(i-1), PY(pts[j][i-1] -> simple), PX(i), PY(pts[j][i] -> simple), ptsColor[j][i] );
			gdImageLine( im, PX(i-1), PY(pts[j][i-1] -> simple)+1, PX(i), PY(pts[j][i] -> simple)+1, ptsColor[j][i] );
			}
		    else
			{
			if( pts[j][i-1] -> simple != DBL_MIN )
			    gdImageSetPixel( im, PX(i-1), PY(pts[j][i-1] -> simple), ptsColor[j][i] );
			if( pts[j][i] -> simple != DBL_MIN )
			    gdImageSetPixel( im, PX(i), PY(pts[j][i] -> simple), ptsColor[j][i] );
			}
	    break;

	case GDC_3DLINE:
	case GDC_3DCOMBO_LINE_BAR:
	case GDC_3DCOMBO_LINE_AREA:
	    {
	    int	y1[nrseries],
		y2[nrseries];

	    for( i=1; i<nrpoints; ++i )
		{
		if( av -> gdccht_stack_type == GDC_STACK_BEHIND )
		    {
		    for( j=nrseries-1; j>=0; --j )
			if( pts[j][i-1] -> simple != DBL_MIN &&
			    pts[j][i] -> simple   != DBL_MIN )
			    {
			    setno = j;
			    y1[j] = PY(pts[j][i-1] -> simple);
			    y2[j] = PY(pts[j][i] -> simple);

			    draw_3d_line( im,
					  PY(0),
					  PX(i-1), PX(i), 
					  &(y1[j]), &(y2[j]),
					  xdepth_3D, ydepth_3D,
					  1,
					  &(ptsColor[j][i]),
					  &(ptsColorShd[j][i]) );
			    setno = 0;
			    }
		    }
		else
		if( av -> gdccht_stack_type == GDC_STACK_BESIDE )
		    {
		    int	    set;
		    int	    clr[nrseries],
			    clrshd[nrseries];
		    double   usey1 = 0.0,
			    usey2 = 0.0;
		    for( j=0,set=0; j<nrseries; ++j )
			if( pts[j][i-1] -> simple != DBL_MIN &&
			    pts[j][i] -> simple   != DBL_MIN )
			    {
				usey1 = pts[j][i-1] -> simple;
				usey2 = pts[j][i] -> simple;
			    y1[set]     = PY(usey1);
			    y2[set]     = PY(usey2);
			    clr[set]    = ptsColor[j][i];
			    clrshd[set] = ptsColorShd[j][i];	/* fred */
			    ++set;
			    }
		    draw_3d_line( im,
			  PY(0),
			  PX(i-1), PX(i), 
			  y1, y2,
			  xdepth_3D, ydepth_3D,
			  set,
			  clr,
			  clrshd );
		    }
		}
	    }
	    break;

	case GDC_AREA:
	case GDC_3DAREA:
	  switch( av -> gdccht_stack_type )
	    {
	    case GDC_STACK_BESIDE:				// behind w/o depth
		for( j=nrseries-1; j>=0; --j )			// back sets 1st  (setno = 0)
		    for( i=1; i<nrpoints; ++i )
			if( pts[j][i-1] -> simple != DBL_MIN && pts[j][i] -> simple != DBL_MIN )
			    draw_3d_area( im, PX(i-1), PX(i),
					     PY(0), PY(pts[j][i-1] -> simple), PY(pts[j][i] -> simple),
					     xdepth_3D, ydepth_3D,
					     ptsColor[j][i],
                                             threeD? ptsColorShd[j][i]: ptsColor[j][i] );
		break;

	    case GDC_STACK_BEHIND:
	    default:
		for( setno=nrseries-1; setno>=0; --setno )	// back sets first   PX, PY depth
		    for( i=1; i<nrpoints; ++i )
			if( pts[setno][i-1] -> simple != DBL_MIN && pts[setno][i] -> simple != DBL_MIN )
			    draw_3d_area( im, PX(i-1), PX(i),
					     PY(0), PY(pts[setno][i-1] -> simple), PY(pts[setno][i] -> simple),
					     xdepth_3D, ydepth_3D,
					     ptsColor[setno][i],
                                             threeD? ptsColorShd[setno][i]: ptsColor[setno][i] );
		setno = 0;
	    }
	    break;

	case GDC_3DHLC:
	case GDC_3DCOMBO_HLC_BAR:
	case GDC_3DCOMBO_HLC_AREA:
	    {
	    gdPoint     poly[4];
	    for( j=num_hlc_sets-1; j>=0; --j )
	     {
	     for( i=1; i<nrpoints+1; ++i )
		 if( pts[j][i-1] -> close != DBL_MIN )
		     {
		     if( (av -> gdccht_HLC_cap_width > 0) &&	    // bottom half of 'I'
			 pts[j][i-1] -> low != DBL_MIN )
			 {
			 SET_3D_POLY( poly, PX(i-1)-hlf_hlccapwdth, PX(i-1)+hlf_hlccapwdth,
					    PY(pts[j][i-1] -> low), PY(pts[j][i-1] -> low),
					    xdepth_3D, ydepth_3D );
			 gdImageFilledPolygon( im, poly, 4, ptsColor[j][i-1] );
			 gdImagePolygon( im, poly, 4, ptsColorShd[j][i-1] );
			 }
								     // all HLC have vert line
		     if( pts[j][i-1] -> low != DBL_MIN )
			 {					    // bottom 'half'
			 SET_3D_POLY( poly, PX(i-1), PX(i-1),
					    PY(pts[j][i-1] -> low), PY(pts[j][i-1] -> close),
					    xdepth_3D, ydepth_3D );
			 gdImageFilledPolygon( im, poly, 4, ptsColor[j][i-1] );
			 gdImagePolygon( im, poly, 4, ptsColorShd[j][i-1] );
			 }
		     if( pts[j][i-1] -> high != DBL_MIN )
			 {					    // top 'half'
			 SET_3D_POLY( poly, PX(i-1), PX(i-1),
					    PY(pts[j][i-1] -> close), PY(pts[j][i-1] -> high),
					    xdepth_3D, ydepth_3D );
			 gdImageFilledPolygon( im, poly, 4, ptsColor[j][i-1] );
			 gdImagePolygon( im, poly, 4, ptsColorShd[j][i-1] );
			 }
								    // line at close
		     gdImageLine( im, PX(i-1),           PY(pts[j][i-1] -> close),
				      PX(i-1)+xdepth_3D, PY(pts[j][i-1] -> close)-ydepth_3D,
				      ptsColorShd[j][i-1] );
								 // top half 'I'
		     if( !( (av -> gdccht_HLC_style == GDC_HLC_DIAMOND) &&
			    (PY(pts[j][i-1] -> high) > PY(pts[j][i-1] -> close)-hlf_hlccapwdth) ) &&
			 pts[j][i-1] -> high != DBL_MIN )
			 if( av -> gdccht_HLC_cap_width > 0 )
			     {
			     SET_3D_POLY( poly, PX(i-1)-hlf_hlccapwdth, PX(i-1)+hlf_hlccapwdth,
						PY(pts[j][i-1] -> high), PY(pts[j][i-1] -> high),
						xdepth_3D, ydepth_3D );
			     gdImageFilledPolygon( im, poly, 4, ptsColor[j][i-1] );
			     gdImagePolygon( im, poly, 4, ptsColorShd[j][i-1] );
			     }

		     if( i < nrpoints &&
			 pts[j][i] -> close != DBL_MIN )
			 {
			 if( av -> gdccht_HLC_style == GDC_HLC_HORIZ_CONNECTED )	/* line from prev close */
			     {
			     SET_3D_POLY( poly, PX(i-1), PX(i),
						PY(pts[j][i-1] -> close), PY(pts[j][i-1] -> close),
						xdepth_3D, ydepth_3D );
			     gdImageFilledPolygon( im, poly, 4, ptsColor[j][i] );
			     gdImagePolygon( im, poly, 4, ptsColorShd[j][i] );
			     }
			 else if( av -> gdccht_HLC_style == GDC_HLC_CONNECTED )   /* thin connecting line */
			     {
			     int    y1 = PY(pts[j][i-1] -> close),
				 y2 = PY(pts[j][i] -> close);
			     draw_3d_line( im,
					   PY(0),
					   PX(i-1), PX(i),
					   &y1, &y2,			// rem only 1 set
					   xdepth_3D, ydepth_3D,
					   1,
					   &(ptsColor[j][i]),
					   &(ptsColorShd[j][i]) );
								     // edge font of it
			     gdImageLine( im, PX(i-1), PY(pts[j][i-1] -> close),
					      PX(i), PY(pts[j][i] -> close),
					      ptsColorShd[j][i] );
			     }
								     // top half 'I' again
			 if( PY(pts[j][i-1] -> close) <= PY(pts[j][i] -> close) &&
			     pts[j][i-1] -> high != DBL_MIN  )
			     if( av -> gdccht_HLC_cap_width > 0 )
				 {
				 SET_3D_POLY( poly, PX(i-1)-hlf_hlccapwdth, PX(i-1)+hlf_hlccapwdth,
						    PY(pts[j][i-1] -> high), PY(pts[j][i-1] -> high),
						    xdepth_3D, ydepth_3D );
				 gdImageFilledPolygon( im, poly, 4, ptsColor[j][i-1] );
				 gdImagePolygon( im, poly, 4, ptsColorShd[j][i-1] );
				 }
			 }
		     if( av -> gdccht_HLC_style == GDC_HLC_DIAMOND )
			 {				    // front
			 poly[0].x = PX(i-1)-hlf_hlccapwdth;
			  poly[0].y = PY(pts[j][i-1] -> close);
			 poly[1].x = PX(i-1);
			  poly[1].y = PY(pts[j][i-1] -> close)+hlf_hlccapwdth;
			 poly[2].x = PX(i-1)+hlf_hlccapwdth;
			  poly[2].y = PY(pts[j][i-1] -> close);
			 poly[3].x = PX(i-1);
			  poly[3].y = PY(pts[j][i-1] -> close)-hlf_hlccapwdth;
			 gdImageFilledPolygon( im, poly, 4, ptsColor[j][i-1] );
			 gdImagePolygon( im, poly, 4, ptsColorShd[j][i-1] );
							     // bottom side
			 SET_3D_POLY( poly, PX(i-1), PX(i-1)+hlf_hlccapwdth,
					    PY(pts[j][i-1] -> close)+hlf_hlccapwdth,
						     PY(pts[j][i-1] -> close),
					    xdepth_3D, ydepth_3D );
			 gdImageFilledPolygon( im, poly, 4, ptsColorShd[j][i-1] );
			 // gdImagePolygon( im, poly, 4, ptsColor[j][i-1] );
							     // top side
			 SET_3D_POLY( poly, PX(i-1), PX(i-1)+hlf_hlccapwdth,
					    PY(pts[j][i-1] -> close)-hlf_hlccapwdth,
						     PY(pts[j][i-1] -> close),
					    xdepth_3D, ydepth_3D );
			 gdImageFilledPolygon( im, poly, 4, ptsColor[j][i-1] );
			 gdImagePolygon( im, poly, 4, ptsColorShd[j][i-1] );
			 }
		     }
	     }
	    }
	    break;

	case GDC_HLC:
	case GDC_COMBO_HLC_BAR:
	case GDC_COMBO_HLC_AREA:
	    for( j=num_hlc_sets-1; j>=0; --j )
		{
		for( i=0; i<nrpoints; ++i )
		    if( pts[j][i] -> close != DBL_MIN )
			{					    /* all HLC have vert line */
			if( pts[j][i] -> low != DBL_MIN )
			    gdImageLine( im, PX(i), PY(pts[j][i] -> close),
					     PX(i), PY(pts[j][i] -> low),
					     ptsColor[j][i] );
			if( pts[j][i] -> high != DBL_MIN )
			    gdImageLine( im, PX(i), PY(pts[j][i] -> high),
					     PX(i), PY(pts[j][i] -> close),
					     ptsColor[j][i] );

			if( av -> gdccht_HLC_cap_width > 0 )
			    {
			    if( pts[j][i] -> low != DBL_MIN )
				gdImageLine( im, PX(i)-hlf_hlccapwdth, PY(pts[j][i] -> low),
						 PX(i)+hlf_hlccapwdth, PY(pts[j][i] -> low),
						 ptsColor[j][i] );
			    if( pts[j][i] -> high != DBL_MIN )
				gdImageLine( im, PX(i)-hlf_hlccapwdth, PY(pts[j][i] -> high),
						 PX(i)+hlf_hlccapwdth, PY(pts[j][i] -> high),
						 ptsColor[j][i] );
			    }
			if( av -> gdccht_HLC_style == GDC_HLC_DIAMOND )
			    {
			    gdPoint         cd[4];

			    cd[0].x = PX(i)-hlf_hlccapwdth; cd[0].y = PY(pts[j][i] -> close);
			    cd[1].x = PX(i);	cd[1].y = PY(pts[j][i] -> close)+hlf_hlccapwdth;
			    cd[2].x = PX(i)+hlf_hlccapwdth; cd[2].y = PY(pts[j][i] -> close);
			    cd[3].x = PX(i);	cd[3].y = PY(pts[j][i] -> close)-hlf_hlccapwdth;
			    gdImageFilledPolygon( im, cd, 4, ptsColor[j][i] );
			    }
			}
		for( i=1; i<nrpoints; ++i )
		    if( pts[j][i-1] -> close != DBL_MIN && pts[j][i] -> close != DBL_MIN )
			{
			if( av -> gdccht_HLC_style == GDC_HLC_HORIZ_CONNECTED )	// line from prev close
				gdImageLine( im, PX(i-1), PY(pts[j][i-1] -> close),
						 PX(i), PY(pts[j][i-1] -> close),
						 ptsColor[j][i] );
			else if( av -> gdccht_HLC_style == GDC_HLC_CONNECTED )	// thin connecting line
			    gdImageLine( im, PX(i-1), PY(pts[j][i-1] -> close),
					     PX(i), PY(pts[j][i] -> close),
					     ptsColor[j][i] );
			}
		}
	    break;
	}
	setno = 0;

    /* ---------- scatter points  over all other plots ---------- */
    /* scatters, by their very nature, don't lend themselves to standard array of points */
    /* also, this affords the opportunity to include scatter points onto any type of chart */
    /* drawing of the scatter point should be an exposed function, so the user can */
    /*  use it to draw a legend, and/or add their own */
    if( av -> gdccht_scatter )
	{
	int	scatter_clr[av -> gdccht_num_scatter_pts];
	gdPoint	ct[3];

	for( i=0; i<av -> gdccht_num_scatter_pts; ++i )
	    {
	    int	    hlf_scatterwdth = (int)( (double)(PX(2)-PX(1))
					     * (((double)((av -> gdccht_scatter+i)->width)/100.0)/2.0) );
	    int	scat_x = PX( (av -> gdccht_scatter+i)->point + (do_bar?1:0) ),
		scat_y = PY( (av -> gdccht_scatter+i)->val );

	    if( (av -> gdccht_scatter+i)->point >= nrpoints ||		    // invalid point
		(av -> gdccht_scatter+i)->point <  0 )
		continue;
	    scatter_clr[i] = GDC_alloc_color(av,im,(av -> gdccht_scatter+i)->color);

	    switch( (av -> gdccht_scatter+i)->ind )
		{
		case GDC_SCATTER_TRIANGLE_UP:
		    ct[0].x = scat_x;
		    ct[0].y = scat_y;
		    ct[1].x = scat_x - hlf_scatterwdth;
		    ct[1].y = scat_y + hlf_scatterwdth;;
		    ct[2].x = scat_x + hlf_scatterwdth;
		    ct[2].y = scat_y + hlf_scatterwdth;
		    if( !do_bar )
			if( (av -> gdccht_scatter+i)->point == 0 )
			    ct[1].x = scat_x;
			else
			if( (av -> gdccht_scatter+i)->point == nrpoints-1 )
			    ct[2].x = scat_x;
		    gdImageFilledPolygon( im, ct, 3, scatter_clr[i] );
		    break;
		case GDC_SCATTER_TRIANGLE_DOWN:
		    ct[0].x = scat_x;
		    ct[0].y = scat_y;
		    ct[1].x = scat_x - hlf_scatterwdth;
		    ct[1].y = scat_y - hlf_scatterwdth;;
		    ct[2].x = scat_x + hlf_scatterwdth;
		    ct[2].y = scat_y - hlf_scatterwdth;
		    if( !do_bar )
			if( (av -> gdccht_scatter+i)->point == 0 )
			    ct[1].x = scat_x;
			else
			if( (av -> gdccht_scatter+i)->point == nrpoints-1 )
			    ct[2].x = scat_x;
		    gdImageFilledPolygon( im, ct, 3, scatter_clr[i] );
		    break;
		}
	    }
	}


    /* box it off */
    /*  after plotting so the outline covers any plot lines */
    if( av -> gdccht_border )
	{
	gdImageLine( im,          PX(0),   PY(lowest), PX(nrpoints-1+(do_bar?2:0)),  PY(lowest), LineColor );

	setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets: nrseries: 1;
	gdImageLine( im,          PX(0),   PY(highest), PX(nrpoints-1+(do_bar?2:0)),  PY(highest), LineColor );
	setno = 0;
	}
    if( av -> gdccht_border )
	{
	int x1, y1, x2, y2;

	x1 = PX(0);
	y1 = PY(highest);
	x2 = PX(nrpoints-1+(do_bar?2:0));
	y2 = PY(lowest);
	gdImageLine( im, x1, PY(lowest), x1, y1, LineColor );

	setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets: nrseries: 1;
	gdImageLine( im, x1, y1, PX(0), PY(highest), LineColor );
	// if( !av -> gdccht_grid || do_vol )			// grid leaves right side Y open
	    {
	    gdImageLine( im, x2, y2, PX(nrpoints-1+(do_bar?2:0)), PY(lowest), LineColor );
	    gdImageLine( im, PX(nrpoints-1+(do_bar?2:0)), PY(lowest),
			     PX(nrpoints-1+(do_bar?2:0)), PY(highest), LineColor );
	    }
	setno = 0;
	}

    if( av -> gdccht_0Shelf && threeD &&				    /* front of 0 shelf */
	  (lowest < 0.0 && highest > 0.0) )
	{
	int x2 = PX( nrpoints-1+(do_bar?2:0) ),
	    y2 = PY( 0 );

	gdImageLine( im, PX(0), PY(0), x2, y2, LineColor );	// front line
	setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets: nrseries:
						 1;		// backmost
								// depth for 3Ds
	gdImageLine( im, x2, y2, PX(nrpoints-1+(do_bar?2:0)), PY(0), LineColor );
	setno = 0;						// set back to foremost
	}

    if( av -> gdccht_annotation )	    /* front half of annotation line */
	{
	int	x1 = PX(av -> gdccht_annotation->point+(do_bar?1:0)),
		y1 = PY(highest);
	int	x2;
							    // front line
	gdImageLine( im, x1, PY(lowest)+1, x1, y1, AnnoteColor );
	if( threeD )
	    {						    // on back plane
	    setno = av -> gdccht_stack_type==GDC_STACK_BEHIND? num_hlc_sets? num_hlc_sets: nrseries: 1;
	    x2 = PX(av -> gdccht_annotation->point+(do_bar?1:0));
							    // prspective line
	    gdImageLine( im, x1, y1, x2, PY(highest), AnnoteColor );
	    }
	else						    // for 3D done with back line
	    {
	    x2 = PX(av -> gdccht_annotation->point+(do_bar?1:0));
	    gdImageLine( im, x1, y1, x1, y1-2, AnnoteColor );
	    }
	/* line-to and note */
	if( *(av -> gdccht_annotation->note) )			    // any note?
	    {
	    if( av -> gdccht_annotation->point >= (nrpoints/2) )	/* note to the left */
		{
		gdImageLine( im, x2,              PY(highest)-2,
				 x2-annote_hgt/2, PY(highest)-2-annote_hgt/2,
				 AnnoteColor );
		GDC_ImageStringNL( im,
				  &GDC_fontc[av -> gdccht_annotation_font],
				  x2-annote_hgt/2-1-annote_len - 1,
				  PY(highest)-annote_hgt+1,
				  av -> gdccht_annotation->note,
				  AnnoteColor,
				  GDC_JUSTIFY_RIGHT );
		}
	    else						/* note to right */
		{
		gdImageLine( im, x2,              PY(highest)-2,
				 x2+annote_hgt/2, PY(highest)-2-annote_hgt/2,
				 AnnoteColor );
		GDC_ImageStringNL( im,
				  &GDC_fontc[av -> gdccht_annotation_font],
				  x2+annote_hgt/2+1 + 1,
				  PY(highest)-annote_hgt+1,
				  av -> gdccht_annotation->note,
				  AnnoteColor,
				  GDC_JUSTIFY_LEFT );
		}
	    }
	setno = 0;
	}

    if( av -> gdc_generate_image )
	image_buffer = gdImagePngPtr(im,buffer_size);

    GDC_save_or_destroy_image(av,im);

    return image_buffer;
}
