/*
 * This files defines the public interface to the charting library.
 */


#ifndef _GDCHART_H
#define _GDCHART_H

#include <stddef.h>
#include <math.h>

#ifdef __cplusplus
extern "C" {
#endif


#ifndef TRUE
#define	TRUE		1
#define	FALSE		0
#endif


#define GDC_NOCOLOR	0x1000000L
#define GDC_DFLTCOLOR	0x2000000L


/* Ordered by size. */

enum GDC_font_size {
	GDC_DEFAULT_FONT = -1,
	GDC_pad = 0,
	GDC_TINY = 1,
	GDC_SMALL = 2,
	GDC_MEDBOLD = 3,
	GDC_LARGE = 4,
	GDC_GIANT = 5,
	GDC_numfonts = 6		/* GDC[PIE]_fontc depend on this. */
};


typedef enum {
	GDC_LINE,
	GDC_AREA,
	GDC_BAR,
	GDC_HLC,
	GDC_COMBO_LINE_BAR,
	GDC_COMBO_LINE_AREA,
	GDC_COMBO_HLC_BAR,
	GDC_COMBO_HLC_AREA,
	GDC_3DLINE,
	GDC_3DAREA,
	GDC_3DBAR,
	GDC_3DHLC,
	GDC_3DCOMBO_LINE_BAR,
	GDC_3DCOMBO_LINE_AREA,
	GDC_3DCOMBO_HLC_BAR,
	GDC_3DCOMBO_HLC_AREA
} GDC_CHART_T;


typedef enum {
	GDC_STACK_BEHIND,
	GDC_STACK_BESIDE,
	GDC_STACK_LAYERED
} GDC_STACK_T;


typedef enum {
	GDC_HLC_DIAMOND,		/* A diamond at the close point. */
	GDC_HLC_HORIZ_CONNECTED,	/* A horizontal line from the close point. */
	GDC_HLC_CONNECTED,		/* A line between close points. */
} GDC_HLC_STYLE_T;


typedef enum {
	GDC_SCATTER_TRIANGLE_DOWN,
	GDC_SCATTER_TRIANGLE_UP
} GDC_SCATTER_IND_T;


typedef enum {
	GDC_3DPIE,
	GDC_2DPIE
} GDCPIE_TYPE;


typedef enum {
	GDCPIE_PCT_NONE,
	GDCPIE_PCT_ABOVE,
	GDCPIE_PCT_BELOW,
	GDCPIE_PCT_RIGHT,
	GDCPIE_PCT_LEFT
} GDCPIE_PCT_TYPE;


/* Only one annotation allowed. */

typedef struct {
	int		point;		/* The data point to annotate. */
	unsigned long	color;		/* The color to use. */
	char		*note;		/* The text of the annotation. */
} GDC_ANNOTATION_T;


typedef struct {
	int		point;		/* 0 <= point < num_points */
	double		val;
	unsigned	width;		/* % (1-100) */
	unsigned long	color;
	GDC_SCATTER_IND_T	ind;
} GDC_SCATTER_T;


/* This defines a data point for a graph. */

typedef struct {
	double			simple;	/* The simple value. */
	double			high;	/* The high value. */
	double			low;	/* The low value. */
	double			close;	/* The close value. */
	unsigned long		color;	/* The color of the point. */
} GDCCHT_DataPoint;


/* This defines a data point for a pie chart (ie. a slice). */

typedef struct {
	double			value;	/* The value. */
	unsigned long		color;	/* The color of the slice. */
	char			*label;	/* The optional label. */
	int			offset;	/* The optional slice offset. */
} GDCPIE_DataPoint;


/* This defines the different attribute values that can be set for a chart. */

typedef struct {
	char			gdc_generate_image;
	char			gdc_hold_img;
	void			*gdc_image;
	unsigned long		gdc_BGColor;
	char			gdc_transparent_bg;
	char			*gdc_title;
	enum GDC_font_size	gdc_title_size;
	unsigned long		gdc_TitleColor;
	enum GDC_font_size	gdc_label_size;
	unsigned long		gdc_LabelColor;
	unsigned long		gdc_LineColor;
	int			gdc_3d_angle;
	int			gdc_3d_depth;

	unsigned long		gdcpie_EdgeColor;
	int			gdcpie_label_dist;
	char			gdcpie_label_line;
	GDCPIE_PCT_TYPE		gdcpie_percent_labels;
	char			gdcpie_other_threshold;

	GDC_STACK_T		gdccht_stack_type;
	unsigned		gdccht_bar_width;
	char			gdccht_0Shelf;
	char			gdccht_grid;
	unsigned long		gdccht_GridColor;
	char			gdccht_border;
	char			gdccht_xaxis;
	char			gdccht_yaxis;
	char			gdccht_yaxis2;
	enum GDC_font_size	gdccht_xaxisfont_size;
	enum GDC_font_size	gdccht_yaxisfont_size;
	char			*gdccht_xtitle;
	char			*gdccht_ytitle;
	char			*gdccht_ytitle2;
	enum GDC_font_size	gdccht_xtitle_size;
	enum GDC_font_size	gdccht_ytitle_size;
	char			*gdccht_ylabel_fmt;
	char			*gdccht_ylabel2_fmt;
	unsigned long		gdccht_XTitleColor;
	unsigned long		gdccht_YTitleColor;
	unsigned long		gdccht_YTitle2Color;
	unsigned long		gdccht_XLabelColor;
	unsigned long		gdccht_YLabelColor;
	unsigned long		gdccht_YLabel2Color;
	GDC_ANNOTATION_T	*gdccht_annotation;
	enum GDC_font_size	gdccht_annotation_font;
	int			gdccht_xlabel_density;
	int			gdccht_ylabel_density;
	double			gdccht_requested_ymin;
	double			gdccht_requested_ymax;
	double			gdccht_requested_yinterval;
	GDC_HLC_STYLE_T		gdccht_HLC_style;
	unsigned		gdccht_HLC_cap_width;
	int			gdccht_num_scatter_pts;
	GDC_SCATTER_T		*gdccht_scatter;
} GDC_AttributeValues;


void *GDC_out_graph(int,int,int *,GDC_CHART_T,GDC_AttributeValues *,int,char **,int,GDCCHT_DataPoint *,GDCCHT_DataPoint *);
void *GDC_out_pie(int,int,int *,GDCPIE_TYPE,GDC_AttributeValues *,int,GDCPIE_DataPoint *);
void GDC_destroy_image(void *);
void GDC_set_defaults(GDC_AttributeValues *);


#ifdef __cplusplus
}
#endif

#endif
