/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the Graph class.
 */


#include <stddef.h>
#include <float.h>

#include "gdchart.h"
#include "pymodint.h"


#define	isHLC(bgpv)	((bgpv) -> high != DBL_MIN ||	\
			 (bgpv) -> low != DBL_MIN ||	\
			 (bgpv) -> close != DBL_MIN)


static int graphCtor(pymodPyBase *,PyObject *,PyObject *);
static PyObject *graphPlot(pymodPyBase *,PyObject *);


/*****************************************************************************/
/* Describe the Graph class.                                                 */
/*****************************************************************************/

/* The wrapped toolkit object. */

static PyTypeObject graphType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"__vpGraph__",
	sizeof (pymodPyGraph),
	0,
	pymodDeallocGraph,
};

/* The methods of this class. */
 
static PyMethodDef graphMethods[] = {
	{"plot", (PyCFunction)graphPlot, METH_VARARGS, },
	{NULL}
};

/* The enums of this class. */
 
static enumValueDef graphEnumValuesGraphType[] = {
	{"Bar", PYMOD_GRAPH_GRAPHTYPE_BAR, },
	{"Area", PYMOD_GRAPH_GRAPHTYPE_AREA, },
	{"Line", PYMOD_GRAPH_GRAPHTYPE_LINE, },
	{"HLC", PYMOD_GRAPH_GRAPHTYPE_HLC, },
	{"LineAndBar", PYMOD_GRAPH_GRAPHTYPE_LINEANDBAR, },
	{"LineAndArea", PYMOD_GRAPH_GRAPHTYPE_LINEANDAREA, },
	{"HLCAndBar", PYMOD_GRAPH_GRAPHTYPE_HLCANDBAR, },
	{"HLCAndArea", PYMOD_GRAPH_GRAPHTYPE_HLCANDAREA, },
	{"Bar3D", PYMOD_GRAPH_GRAPHTYPE_BAR3D, },
	{"Area3D", PYMOD_GRAPH_GRAPHTYPE_AREA3D, },
	{"Line3D", PYMOD_GRAPH_GRAPHTYPE_LINE3D, },
	{"HLC3D", PYMOD_GRAPH_GRAPHTYPE_HLC3D, },
	{"LineAndBar3D", PYMOD_GRAPH_GRAPHTYPE_LINEANDBAR3D, },
	{"LineAndArea3D", PYMOD_GRAPH_GRAPHTYPE_LINEANDAREA3D, },
	{"HLCAndBar3D", PYMOD_GRAPH_GRAPHTYPE_HLCANDBAR3D, },
	{"HLCAndArea3D", PYMOD_GRAPH_GRAPHTYPE_HLCANDAREA3D, },
	{NULL}
};
 
static PyTypeObject graphEnumTypeGraphType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"GraphType",
	sizeof (enumObject),
	0,
	0,
	0,
	0,
	0,
	0,
	(reprfunc)pymodEnumRepr,
};
 
enumDef pymodEnumGraphGraphType = {
	&graphEnumTypeGraphType,
	graphEnumValuesGraphType
};

static enumDef *graphEnums[] = {
	&pymodEnumGraphGraphType,
	NULL
};

/* The properties of this class. */

propDef pymodPropGraphComboSeries = {
	"comboSeries",
	instanceProp,
	offsetof(pymodPyGraph,graph.comboSeries),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	NULL,
	&pymodClassGraphSeries,
};

propDef pymodPropGraphComboTitle = {
	"comboTitle",
	stringProp,
	offsetof(pymodPyGraph,graph.comboTitle),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
};

propDef pymodPropGraphComboTitleColor = {
	"comboTitleColor",
	instanceProp,
	offsetof(pymodPyGraph,graph.comboTitleColor),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	NULL,
	&pymodClassColor,
};

propDef pymodPropGraphGraphType = {
	"graphType",
	enumProp,
	offsetof(pymodPyGraph,graph.graphType),
	0,
	{0, },
	{0, },
	&pymodEnumGraphGraphType,
};

propDef pymodPropGraphLabels = {
	"labels",
	stringProp,
	offsetof(pymodPyGraph,graph.labels),
	PROP_FLAG_LIST,
	{PROP_VALID_NONE_OK|PROP_VALID_EMPTY_OK, },
};

propDef pymodPropGraphPlotColor = {
	"plotColor",
	instanceProp,
	offsetof(pymodPyGraph,graph.plotColor),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	NULL,
	&pymodClassColor,
};

propDef pymodPropGraphSeries = {
	"series",
	instanceProp,
	offsetof(pymodPyGraph,graph.series),
	PROP_FLAG_LIST,
	{PROP_VALID_EMPTY_OK, },
	{0, },
	NULL,
	&pymodClassGraphSeries,
};

propDef pymodPropGraphXTitle = {
	"xTitle",
	stringProp,
	offsetof(pymodPyGraph,graph.xTitle),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
};

propDef pymodPropGraphXTitleColor = {
	"xTitleColor",
	instanceProp,
	offsetof(pymodPyGraph,graph.xTitleColor),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	NULL,
	&pymodClassColor,
};

propDef pymodPropGraphYTitle = {
	"yTitle",
	stringProp,
	offsetof(pymodPyGraph,graph.yTitle),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
};

propDef pymodPropGraphYTitleColor = {
	"yTitleColor",
	instanceProp,
	offsetof(pymodPyGraph,graph.yTitleColor),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	NULL,
	&pymodClassColor,
};

static classPropDef graphProperties[] = {
	{&pymodPropControlParent, },
	{&pymodPropGraphGraphType, pymodNullHandler, },
	{&pymodPropChartHeight, pymodNullHandler, },
	{&pymodPropChartWidth, pymodNullHandler, },
	{&pymodPropControlColumn, },
	{&pymodPropGraphComboSeries, pymodNullHandler, },
	{&pymodPropGraphComboTitle, pymodNullHandler, },
	{&pymodPropGraphComboTitleColor, pymodNullHandler, },
	{&pymodPropChartLabelColor, pymodNullHandler, },
	{&pymodPropGraphLabels, pymodNullHandler, },
	{&pymodPropGraphPlotColor, pymodNullHandler, },
	{&pymodPropControlRow, },
	{&pymodPropGraphSeries, pymodNullHandler, },
	{&pymodPropChartTitle, pymodNullHandler, },
	{&pymodPropChartTitleColor, pymodNullHandler, },
	{&pymodPropGraphXTitle, pymodNullHandler, },
	{&pymodPropGraphXTitleColor, pymodNullHandler, },
	{&pymodPropGraphYTitle, pymodNullHandler, },
	{&pymodPropGraphYTitleColor, pymodNullHandler, },
	{NULL}
};


/* The class definition. */
 
classDef pymodClassGraph = {
	"Graph",
	graphProperties,
	graphMethods,
	graphEnums,
	NULL,
	graphCtor,
	&graphType,
};


/*
 * The class ctor.
 */

static int graphCtor(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	pymodDataGraph *gd = &((pymodPyGraph *)self) -> graph;

	if (pymodParseArgs(&pymodClassGraph,self,args,kwds,graphProperties,"c") < 0)
		return -1;

	if (pymodInitControl1(self) < 0)
		return -1;

	/*
	 * Call the driver.
	 *
	 * parent is the parent toolkit object.
	 */

	if ((gd -> chart.control.base.tkPtr = (*((pymodGraphDrivers *)pymodClassGraph.drivers) -> ctor)(
			(PyObject *)self,
			gd -> chart.control.parent -> container.control.base.tkPtr,
			gd -> chart.width,
			gd -> chart.height)) == NULL)
	{
		pymodBadToolkitCtor(pymodClassGraph.name);
		return -1;
	}

	pymodInitControl2(self);

	return 0;
}


/*
 * The method plot.
 */

static PyObject *graphPlot(pymodPyBase *self,PyObject *args)
{
	char **labels;
	int image_size, gtype, nrseries, nrpoints, iscombo, ishlc, i;
	void *image;
	pymodPyGraph *gw;
	GDC_AttributeValues av;
	PyObject *slist, *res;
	GDCCHT_DataPoint *sarray, *comboSeries, *sp;

	if (pymodParseArgs(&pymodClassGraph,self,args,NULL,NULL,
				"mplot",
				&gw) < 0)
		return NULL;

	/* Get the graph type. */

	iscombo = ishlc = 0;

	switch (gw -> graph.graphType)
	{
	case PYMOD_GRAPH_GRAPHTYPE_BAR:
		gtype = GDC_BAR;
		av.gdccht_stack_type = GDC_STACK_BESIDE;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_AREA:
		gtype = GDC_AREA;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_LINE:
		gtype = GDC_LINE;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_HLC:
		gtype = GDC_HLC;
		ishlc = 1;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_LINEANDBAR:
		gtype = GDC_COMBO_LINE_BAR;
		iscombo = 1;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_LINEANDAREA:
		gtype = GDC_COMBO_LINE_AREA;
		iscombo = 1;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_HLCANDBAR:
		gtype = GDC_COMBO_HLC_BAR;
		iscombo = ishlc = 1;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_HLCANDAREA:
		gtype = GDC_COMBO_HLC_AREA;
		iscombo = ishlc = 1;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_BAR3D:
		gtype = GDC_3DBAR;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_AREA3D:
		gtype = GDC_3DAREA;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_LINE3D:
		gtype = GDC_3DLINE;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_HLC3D:
		gtype = GDC_3DHLC;
		ishlc = 1;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_LINEANDBAR3D:
		gtype = GDC_3DCOMBO_LINE_BAR;
		iscombo = 1;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_LINEANDAREA3D:
		gtype = GDC_3DCOMBO_LINE_AREA;
		iscombo = 1;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_HLCANDBAR3D:
		gtype = GDC_3DCOMBO_HLC_BAR;
		iscombo = ishlc = 1;
		break;

	case PYMOD_GRAPH_GRAPHTYPE_HLCANDAREA3D:
		gtype = GDC_3DCOMBO_HLC_AREA;
		iscombo = ishlc = 1;
		break;
	}

	/* Find out how many series, and the length of the longest series. */

	nrseries = nrpoints = 0;

	if ((slist = gw -> graph.series) != NULL)
	{
		int spts;
		pymodPyGraphSeries *gsw;

		nrseries = PyList_GET_SIZE(slist);

		for (i = 0; i < nrseries; ++i)
		{
			gsw = (pymodPyGraphSeries *)PyList_GET_ITEM(slist,i);

			if (gsw -> graphSeries.values != NULL)
			{
				spts = PyList_GET_SIZE(gsw -> graphSeries.values);

				if (spts > nrpoints)
					nrpoints = spts;
			}
		}

		if (iscombo && (gsw = gw -> graph.comboSeries) != NULL && gsw -> graphSeries.values != NULL)
		{
			spts = PyList_GET_SIZE(gsw -> graphSeries.values);

			if (spts > nrpoints)
				nrpoints = spts;
		}
	}

	/* Check we have something to plot. */

	if (nrseries == 0 || nrpoints == 0)
	{
		PyErr_Format(PyExc_ValueError,"There is no data to plot");
		return NULL;
	}

	/*
	 * Initialise all of the pointers to dynamic memory so we can easily
	 * unravel them.
	 */

	res = NULL;
	labels = NULL;
	comboSeries = NULL;
	sarray = NULL;

	/* Create the list of labels. */

	if (gw -> graph.labels != NULL)
	{
		if ((labels = Py_Malloc(sizeof (char *) * nrpoints)) == NULL)
			goto freemem;

		for (i = 0; i < nrpoints; ++i)
		{
			PyObject *lab;

			if (i < PyList_GET_SIZE(gw -> graph.labels) &&
			    (lab = PyList_GET_ITEM(gw -> graph.labels,i)) != NULL)
				labels[i] = PyString_AS_STRING(lab);
			else
				labels[i] = NULL;
		}
	}

	/* Convert the combo series if one is needed. */

	if (iscombo)
	{
		if ((comboSeries = Py_Malloc(sizeof (GDCCHT_DataPoint) * nrpoints)) == NULL)
			goto freemem;

		for (i = 0; i < nrpoints; ++i)
		{
			int slen;
			pymodPyGraphSeries *gsw;
			pymodPyGraphPoint *gpw;
			GDCCHT_DataPoint *pnt = &comboSeries[i];

			if ((gsw = gw -> graph.comboSeries) != NULL && gsw -> graphSeries.values != NULL)
				slen = PyList_GET_SIZE(gsw -> graphSeries.values);
			else
				slen = 0;

			if (i < slen &&
			    (gpw = (pymodPyGraphPoint *)PyList_GET_ITEM(gsw -> graphSeries.values,i)) != NULL)
			{
				pymodPyColor *col;

				if (isHLC(&gpw -> graphPoint.value))
				{
					PyErr_Format(PyExc_TypeError,"The %s property cannot contain HLC values",pymodPropGraphComboSeries.name);

					goto freemem;
				}

				pnt -> simple = gpw -> graphPoint.value.simple;

				if ((col = gpw -> graphPoint.color) == NULL)
					if ((col = gsw -> graphSeries.color) == NULL)
						col = gw -> graph.plotColor;

				pnt -> color = pymodChartColor(col);
			}
			else
			{
				/* Extend the series with empty points. */

				pnt -> simple = DBL_MIN;
				pnt -> color = GDC_DFLTCOLOR;
			}
		}
	}

	/* Convert the main series. */

	if ((sarray = Py_Malloc(sizeof (GDCCHT_DataPoint) * nrseries * nrpoints)) == NULL)
		goto freemem;

	sp = sarray;

	for (i = 0; i < nrseries; ++i)
	{
		int j;
		pymodPyGraphSeries *gsw;

		gsw = (pymodPyGraphSeries *)PyList_GET_ITEM(gw -> graph.series,i);

		for (j = 0; j < nrpoints; ++j)
		{
			int slen;
			pymodPyGraphPoint *gpw;

			if (gsw -> graphSeries.values != NULL)
				slen = PyList_GET_SIZE(gsw -> graphSeries.values);
			else
				slen = 0;

			if (i < slen &&
			    (gpw = (pymodPyGraphPoint *)PyList_GET_ITEM(gsw -> graphSeries.values,j)) != NULL)
			{
				pymodPyColor *col;

				if (!ishlc && isHLC(&gpw -> graphPoint.value))
				{
					PyErr_Format(PyExc_TypeError,"With this graph type the %s property cannot contain HLC values",pymodPropGraphSeries.name);

					goto freemem;
				}

				if (ishlc && gpw -> graphPoint.value.simple != DBL_MIN)
				{
					PyErr_Format(PyExc_TypeError,"With this graph type the %s property must contain HLC values",pymodPropGraphSeries.name);

					goto freemem;
				}

				sp -> simple = gpw -> graphPoint.value.simple;
				sp -> high = gpw -> graphPoint.value.high;
				sp -> low = gpw -> graphPoint.value.low;
				sp -> close = gpw -> graphPoint.value.close;

				if ((col = gpw -> graphPoint.color) == NULL)
					if ((col = gsw -> graphSeries.color) == NULL)
						col = gw -> graph.plotColor;

				sp -> color = pymodChartColor(col);
			}
			else
			{
				/* Extend the series with empty points. */

				sp -> simple = DBL_MIN;
				sp -> high = DBL_MIN;
				sp -> low = DBL_MIN;
				sp -> close = DBL_MIN;
				sp -> color = GDC_DFLTCOLOR;
			}

			++sp;
		}
	}

	/* Set the other properties. */

	GDC_set_defaults(&av);

	av.gdc_BGColor = pymodChartColor(gw -> graph.chart.control.background);

	av.gdc_LabelColor = pymodChartColor(gw -> graph.chart.labelColor);

	if (gw -> graph.chart.title != NULL)
		av.gdc_title = PyString_AS_STRING(gw -> graph.chart.title);

	av.gdc_TitleColor = pymodChartColor(gw -> graph.chart.titleColor);

	if (gtype == GDC_BAR)
		av.gdccht_stack_type = GDC_STACK_BESIDE;

	if (gw -> graph.comboTitle != NULL)
		av.gdccht_ytitle2 = PyString_AS_STRING(gw -> graph.comboTitle);

	av.gdccht_YTitle2Color = pymodChartColor(gw -> graph.comboTitleColor);

	if (gw -> graph.xTitle != NULL)
		av.gdccht_xtitle = PyString_AS_STRING(gw -> graph.xTitle);

	av.gdccht_XTitleColor = pymodChartColor(gw -> graph.xTitleColor);

	if (gw -> graph.yTitle != NULL)
		av.gdccht_ytitle = PyString_AS_STRING(gw -> graph.yTitle);

	av.gdccht_YTitleColor = pymodChartColor(gw -> graph.yTitleColor);

	/* Generate the image. */

	image = GDC_out_graph(gw -> graph.chart.width,
			      gw -> graph.chart.height,
			      &image_size,gtype,&av,nrpoints,labels,nrseries,
			      sarray,comboSeries);

	if (image == NULL)
	{
		PyErr_Format(PyExc_SystemError,"Internal error: unable to create graph");
		goto freemem;
	}

	/* Call the driver. */

	(*((pymodGraphDrivers *)pymodClassGraph.drivers) -> plot)(
		gw -> graph.chart.control.base.tkPtr,
		image,
		image_size);

	Py_Free((ANY *)image);

	Py_INCREF(Py_None);
	res = Py_None;

freemem:
	if (labels != NULL)
		Py_Free((ANY *)labels);

	if (comboSeries != NULL)
		Py_Free((ANY *)comboSeries);

	if (sarray != NULL)
		Py_Free((ANY *)sarray);

	return res;
}
