/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module handles the initialisation of the GUI independent code.
 */


#include "pymodint.h"


/*
 * Perform the common Python initialisation for a module.
 */

void pymodInit(char *modname,
		pymodColorDrivers *colorDrivers,
		pymodGridDrivers *gridDrivers,
		pymodHBoxDrivers *hBoxDrivers,
		pymodVBoxDrivers *vBoxDrivers,
		pymodPieChartDrivers *pieChartDrivers,
		pymodGraphDrivers *graphDrivers,
		pymodLabelDrivers *labelDrivers,
		pymodApplicationWindowDrivers *applicationWindowDrivers,
		pymodMessageDrivers *messageDrivers,
		pymodDialogDrivers *dialogDrivers)
{
	static PyMethodDef dummyMethods[] = {
		{NULL}
	};

	static PyMethodDef methods[] = {
		{"__vpInit__", pymodCallCtor, METH_VARARGS,},
		{"__vpGetattr__", pymodCallGetattr, METH_VARARGS,},
		{"__vpSetattr__", pymodCallSetattr, METH_VARARGS,},
		{"connect", pymodConnect, METH_VARARGS,},
		{"disconnect", pymodDisconnect, METH_VARARGS,},
		{NULL}
	};

	PyObject *dictofmods, *mod, *vpmoddict, *meth;
	PyMethodDef *md;

	/* Initialise the Python module. */

	if (Py_InitModule(modname,dummyMethods) == NULL)
		goto pyerror;

	/* Get the dictionary for the vp module. */

	if ((dictofmods = PyImport_GetModuleDict()) == NULL ||
	    (mod = PyDict_GetItemString(dictofmods,"vp")) == NULL ||
	    (vpmoddict = PyModule_GetDict(mod)) == NULL)
		goto pyerror;

	/*
	 * Add the methods to access the C module to vp module rather than this
	 * module so that the vp module doesn't need to know the name of this
	 * module (which will be different for each toolkit implementation).
	 * The more sensible thing to do might be to create everything in C but 
	 * Python insists that __init__ is a PyFunction rather than either a
	 * PyFunction or a PyCFunction.
	 */

	for (md = methods; md -> ml_name != NULL; ++md)
	{
		PyObject *meth;

		if ((meth = PyCFunction_New(md,NULL)) == NULL)
			goto pyerror;

		if (PyDict_SetItemString(vpmoddict,md -> ml_name,meth) != 0)
			goto pyerror;

		Py_DECREF(meth);
	}

	/* Initialise the drivers. */

	pymodClassColor.drivers = (guiDriversDef *)colorDrivers;
	pymodClassGrid.drivers = (guiDriversDef *)gridDrivers;
	pymodClassHBox.drivers = (guiDriversDef *)hBoxDrivers;
	pymodClassVBox.drivers = (guiDriversDef *)vBoxDrivers;
	pymodClassPieChart.drivers = (guiDriversDef *)pieChartDrivers;
	pymodClassGraph.drivers = (guiDriversDef *)graphDrivers;
	pymodClassLabel.drivers = (guiDriversDef *)labelDrivers;
	pymodClassApplicationWindow.drivers = (guiDriversDef *)applicationWindowDrivers;
	pymodClassMessage.drivers = (guiDriversDef *)messageDrivers;
	pymodClassDialog.drivers = (guiDriversDef *)dialogDrivers;

	/* Initialise each of the classes. */

	if (pymodInitClasses(vpmoddict) < 0)
		goto pyerror;

	/* No errors. */

	return;

pyerror:
	pylibException();
}
