/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the Label class.
 */


#include <stddef.h>

#include "pymodint.h"


static int labelCtor(pymodPyBase *,PyObject *,PyObject *);
static int labelSetText(pymodPyBase *);


/*****************************************************************************/
/* Describe the Label class.                                                */
/*****************************************************************************/

/* The wrapped toolkit object. */

static PyTypeObject labelType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"__vpLabel__",
	sizeof (pymodPyLabel),
	0,
	pymodDeallocLabel,
};

/* The properties of this class. */

propDef pymodPropLabelText = {
	"text",
	stringProp,
	offsetof(pymodPyLabel,label.text),
	0,
	{0, },
	{0, },
};

static classPropDef labelProperties[] = {
	{&pymodPropControlParent, },
	{&pymodPropControlColumn, },
	{&pymodPropControlRow, },
	{&pymodPropLabelText, labelSetText, },
	{NULL}
};


/* The class definition. */
 
classDef pymodClassLabel = {
	"Label",
	labelProperties,
	NULL,
	NULL,
	NULL,
	labelCtor,
	&labelType,
};


/*
 * The class ctor.
 */

static int labelCtor(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	pymodDataLabel *ld = &((pymodPyLabel *)self) -> label;

	if (pymodParseArgs(&pymodClassLabel,self,args,kwds,labelProperties,"c") < 0)
		return -1;

	if (pymodInitControl1(self) < 0)
		return -1;

	/*
	 * Call the driver.
	 *
	 * parent is the parent toolkit object.
	 * text is the text of the label.
	 */

	if ((ld -> control.base.tkPtr = (*((pymodLabelDrivers *)pymodClassLabel.drivers) -> ctor)(
			(PyObject *)self,
			ld -> control.parent -> container.control.base.tkPtr,
			PyString_AS_STRING(ld -> text))) == NULL)
	{
		pymodBadToolkitCtor(pymodClassLabel.name);
		return -1;
	}

	pymodInitControl2(self);

	return 0;
}


/*
 * Handle the setting of the text property.
 */

static int labelSetText(pymodPyBase *self)
{
	pymodDataLabel *ld = &((pymodPyLabel *)self) -> label;

	return (*((pymodLabelDrivers *)pymodClassLabel.drivers) -> setText)(
			ld -> control.base.tkPtr,
			PyString_AS_STRING(ld -> text));
}
