/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the Message class.
 */


#include <stddef.h>

#include "pymodint.h"


static PyObject *messageError(pymodPyBase *,PyObject *,PyObject *);
static PyObject *messageInformation(pymodPyBase *,PyObject *,PyObject *);
static PyObject *messageQuestion(pymodPyBase *,PyObject *,PyObject *);
static PyObject *messageSorry(pymodPyBase *,PyObject *,PyObject *);
static PyObject *messageWarning(pymodPyBase *,PyObject *,PyObject *);


/*****************************************************************************/
/* Describe the Message class.                                               */
/*****************************************************************************/

/* The functions of this class. */

static PyMethodDef messageFunctions[] = {
	{"error", (PyCFunction)messageError, METH_VARARGS|METH_KEYWORDS, },
	{"information", (PyCFunction)messageInformation, METH_VARARGS|METH_KEYWORDS, },
	{"question", (PyCFunction)messageQuestion, METH_VARARGS|METH_KEYWORDS, },
	{"sorry", (PyCFunction)messageSorry, METH_VARARGS|METH_KEYWORDS, },
	{"warning", (PyCFunction)messageWarning, METH_VARARGS|METH_KEYWORDS, },
	{NULL, }
};

/* The properties of this class. */

propDef pymodPropMessageText = {
	"text",
	stringProp,
	offsetof(pymodPyMessage,message.text),
	0,
	{0, },
	{0, },
};

static classPropDef messageProperties[] = {
	{&pymodPropMessageText, },
	{NULL}
};

/* The class definition. */

classDef pymodClassMessage = {
	"Message",
	messageProperties,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	messageFunctions,
};


/*
 * The sorry static method.
 */

static PyObject *messageSorry(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	static classPropDef props[] = {
		{&pymodPropMessageText, },
		{&pymodPropControlParent, NULL,
			{PROP_VALID_OVERRIDE|PROP_VALID_NONE_OK, },
			{PROP_VALUE_OVERRIDE|PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, }, },
		{&pymodPropTopLevelTitle, },
		{NULL}
	};

	PyObject *text, *title;
	pymodPyBase *parent;
	struct pylibContext *context;

	if (pymodParseArgs(&pymodClassMessage,self,args,kwds,props,
				"ssorry",
				&text,
				&parent,
				&title) < 0)
		return NULL;

	/*
	 * Call the driver.
	 *
	 * text is the text of the message.
	 * parent is the parent toolkit object, NULL means there is no parent.
	 * title is the text of the dialog title, NULL means use the default.
	 */

	context = pylibGetAndClearContext();

	(*((pymodMessageDrivers *)pymodClassMessage.drivers) -> sorry)(
		PyString_AS_STRING(text),
		getOptTkPtr(parent),
		getOptString(title));

	pylibRestoreContext(context);

	pymodPropsDelete(props,&text,&parent,&title);

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * The error static method.
 */

static PyObject *messageError(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	static classPropDef props[] = {
		{&pymodPropMessageText, },
		{&pymodPropControlParent, NULL,
			{PROP_VALID_OVERRIDE|PROP_VALID_NONE_OK, },
			{PROP_VALUE_OVERRIDE|PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, }, },
		{&pymodPropTopLevelTitle, },
		{NULL}
	};

	PyObject *text, *title;
	pymodPyBase *parent;
	struct pylibContext *context;

	if (pymodParseArgs(&pymodClassMessage,self,args,kwds,props,
				"serror",
				&text,
				&parent,
				&title) < 0)
		return NULL;

	/*
	 * Call the driver.
	 *
	 * text is the text of the message.
	 * parent is the parent toolkit object, NULL means there is no parent.
	 * title is the text of the dialog title, NULL means use the default.
	 */

	context = pylibGetAndClearContext();

	(*((pymodMessageDrivers *)pymodClassMessage.drivers) -> error)(
		PyString_AS_STRING(text),
		getOptTkPtr(parent),
		getOptString(title));

	pylibRestoreContext(context);

	pymodPropsDelete(props,&text,&parent,&title);

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * The information static method.
 */

static PyObject *messageInformation(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	static classPropDef props[] = {
		{&pymodPropMessageText, },
		{&pymodPropControlParent, NULL,
			{PROP_VALID_OVERRIDE|PROP_VALID_NONE_OK, },
			{PROP_VALUE_OVERRIDE|PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, }, },
		{&pymodPropTopLevelTitle, },
		{NULL}
	};

	PyObject *text, *title;
	pymodPyBase *parent;
	struct pylibContext *context;

	if (pymodParseArgs(&pymodClassMessage,self,args,kwds,props,
				"sinformation",
				&text,
				&parent,
				&title) < 0)
		return NULL;

	/*
	 * Call the driver.
	 *
	 * text is the text of the message.
	 * parent is the parent toolkit object, NULL means there is no parent.
	 * title is the text of the dialog title, NULL means use the default.
	 */

	context = pylibGetAndClearContext();

	(*((pymodMessageDrivers *)pymodClassMessage.drivers) -> information)(
		PyString_AS_STRING(text),
		getOptTkPtr(parent),
		getOptString(title));

	pylibRestoreContext(context);

	pymodPropsDelete(props,&text,&parent,&title);

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * The question static method.
 */

static PyObject *messageQuestion(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	static classPropDef props[] = {
		{&pymodPropMessageText, },
		{&pymodPropControlParent, NULL,
			{PROP_VALID_OVERRIDE|PROP_VALID_NONE_OK, },
			{PROP_VALUE_OVERRIDE|PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, }, },
		{&pymodPropTopLevelTitle, },
		{&pymodPropDialogNoText, },
		{&pymodPropDialogYesText, },
		{NULL}
	};

	PyObject *text, *title, *yestext, *notext;
	pymodPyBase *parent;
	struct pylibContext *context;
	int butt;

	if (pymodParseArgs(&pymodClassMessage,self,args,kwds,props,
				"squestion",
				&text,
				&parent,
				&title,
				&notext,
				&yestext) < 0)
		return NULL;

	/*
	 * Call the driver.
	 *
	 * text is the text of the message.
	 * parent is the parent toolkit object, NULL means there is no parent.
	 * title is the text of the dialog title, NULL means use the default.
	 * notext is the text of the No button, NULL means use the default.
	 * yestext is the text of the Yes button, NULL means use the default.
	 *
	 * Returns PYMOD_DIALOG_BUTTON_YES or PYMOD_DIALOG_BUTTON_NO.
	 */

	context = pylibGetAndClearContext();

	butt = (*((pymodMessageDrivers *)pymodClassMessage.drivers) -> question)(
		PyString_AS_STRING(text),
		getOptTkPtr(parent),
		getOptString(title),
		getOptString(notext),
		getOptString(yestext));

	pylibRestoreContext(context);

	pymodPropsDelete(props,&text,&parent,&title,&notext,&yestext);

	return pymodConvertEnum(&pymodEnumDialogButton,butt);
}


/*
 * The warning static method.
 */

static PyObject *messageWarning(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	static int validbutts[] = {
		PYMOD_DIALOG_BUTTON_CONTINUE|PYMOD_DIALOG_BUTTON_CANCEL,
		PYMOD_DIALOG_BUTTON_YES|PYMOD_DIALOG_BUTTON_NO|PYMOD_DIALOG_BUTTON_CANCEL,
		PYMOD_DIALOG_BUTTON_YES|PYMOD_DIALOG_BUTTON_NO,
		0
	};

	static classPropDef props[] = {
		{&pymodPropMessageText, },
		{&pymodPropControlParent, NULL,
			{PROP_VALID_OVERRIDE|PROP_VALID_NONE_OK, },
			{PROP_VALUE_OVERRIDE|PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, }, },
		{&pymodPropTopLevelTitle, },
		{&pymodPropDialogButtons, NULL,
			{PROP_VALID_OVERRIDE, validbutts},
			{PROP_VALUE_OVERRIDE|PROP_VALUE_IS_DEFAULT,
			 PYMOD_DIALOG_BUTTON_CONTINUE|PYMOD_DIALOG_BUTTON_CANCEL}},
		{&pymodPropDialogContinueText, },
		{&pymodPropDialogNoText, },
		{&pymodPropDialogYesText, },
		{NULL}
	};

	PyObject *text, *title, *yestext, *notext, *continuetext;
	pymodPyBase *parent;
	struct pylibContext *context;
	int butt, buttons;

	if (pymodParseArgs(&pymodClassMessage,self,args,kwds,props,
				"swarning",
				&text,
				&parent,
				&title,
				&buttons,
				&continuetext,
				&notext,
				&yestext) < 0)
		return NULL;

	/*
	 * Call the driver.
	 *
	 * text is the text of the message.
	 * parent is the parent toolkit object, NULL means there is no parent.
	 * title is the text of the dialog title, NULL means use the default.
	 * buttons is the set of buttons to display.
	 * continuetext is the text of any No button, NULL means use the
	 * default.
	 * notext is the text of any No button, NULL means use the default.
	 * yestext is the text of any Yes button, NULL means use the default.
	 *
	 * Returns PYMOD_DIALOG_BUTTON_YES, PYMOD_DIALOG_BUTTON_NO,
	 * PYMOD_DIALOG_BUTTON_CONTINUE or PYMOD_DIALOG_BUTTON_CANCEL.
	 */

	context = pylibGetAndClearContext();

	butt = (*((pymodMessageDrivers *)pymodClassMessage.drivers) -> warning)(
		PyString_AS_STRING(text),
		getOptTkPtr(parent),
		getOptString(title),
		buttons,
		getOptString(continuetext),
		getOptString(notext),
		getOptString(yestext));

	pylibRestoreContext(context);

	pymodPropsDelete(props,&text,&parent,&title,&buttons,&continuetext,&notext,&yestext);

	return pymodConvertEnum(&pymodEnumDialogButton,butt);
}
