/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This header files defines the interface between the GUI dependent and
 * independent parts of VP.
 */


#ifndef _PYMOD_H
#define	_PYMOD_H


#include <Python.h>


#ifdef __cplusplus
extern "C" {
#endif


/* The GUI independent representation of a color. */
 
typedef struct {
	unsigned short	red;
	unsigned short	blue;
	unsigned short	green;
	int		bitsPerColor;
} pymodColorDef;


/*****************************************************************************/
/* The Dialog class.                                                         */
/*****************************************************************************/

#define	PYMOD_DIALOG_MODALITY_MODAL		(1 << 0)
#define	PYMOD_DIALOG_MODALITY_NONMODAL		(1 << 1)

#define	PYMOD_DIALOG_BUTTON_OK			(1 << 0)
#define	PYMOD_DIALOG_BUTTON_CANCEL		(1 << 1)
#define	PYMOD_DIALOG_BUTTON_YES			(1 << 2)
#define	PYMOD_DIALOG_BUTTON_NO			(1 << 3)
#define	PYMOD_DIALOG_BUTTON_CONTINUE		(1 << 4)

typedef struct {
	void		(*dtor)(void *);
	void		*(*ctor)(PyObject *,void *,char *,int,char *,int,int,char *,char *);
	void		(*addChild)(void *,void *);
	void		(*hide)(void *);
	void		(*show)(void *);
	int		(*setTitle)(void *,char *);
	int		(*getBackground)(void *,pymodColorDef *);
} pymodDialogDrivers;

void pymodSignalDialogClicked(PyObject *,int);


/*****************************************************************************/
/* The Message class.                                                        */
/*****************************************************************************/

typedef struct {
	void		(*error)(char *,void *,char *);
	void		(*information)(char *,void *,char *);
	int		(*question)(char *,void *,char *,char *,char *);
	void		(*sorry)(char *,void *,char *);
	int		(*warning)(char *,void *,char *,int,char *,char *,char *);
} pymodMessageDrivers;


/*****************************************************************************/
/* The Grid class.                                                           */
/*****************************************************************************/

typedef struct {
	void		(*dtor)(void *);
	void		*(*ctor)(PyObject *,void *);
	void		(*addChild)(void *,void *,int,int);
} pymodGridDrivers;


/*****************************************************************************/
/* The HBox class.                                                           */
/*****************************************************************************/

typedef struct {
	void		(*dtor)(void *);
	void		*(*ctor)(PyObject *,void *);
	void		(*addChild)(void *,void *);
} pymodHBoxDrivers;


/*****************************************************************************/
/* The VBox class.                                                           */
/*****************************************************************************/

typedef struct {
	void		(*dtor)(void *);
	void		*(*ctor)(PyObject *,void *);
	void		(*addChild)(void *,void *);
} pymodVBoxDrivers;


/*****************************************************************************/
/* The ApplicationWindow class.                                              */
/*****************************************************************************/

typedef struct {
	void		(*dtor)(void *);
	void		(*addChild)(void *,void *);
	void		(*update)(void *,void *);
	int		(*getBackground)(void *,pymodColorDef *);
} pymodApplicationWindowDrivers;


/*****************************************************************************/
/* The Label class.                                                          */
/*****************************************************************************/

typedef struct {
	void		(*dtor)(void *);
	void		*(*ctor)(PyObject *,void *,char *);
	int		(*setText)(void *,char *);
} pymodLabelDrivers;


/*****************************************************************************/
/* The PieChart class.                                                       */
/*****************************************************************************/

typedef struct {
	void		(*dtor)(void *);
	void		*(*ctor)(PyObject *,void *,int,int);
	void		(*plot)(void *,void *,int);
} pymodPieChartDrivers;


/*****************************************************************************/
/* The Graph class.                                                          */
/*****************************************************************************/

#define	PYMOD_GRAPH_GRAPHTYPE_BAR		(1 << 0)
#define	PYMOD_GRAPH_GRAPHTYPE_AREA		(1 << 1)
#define	PYMOD_GRAPH_GRAPHTYPE_LINE		(1 << 2)
#define	PYMOD_GRAPH_GRAPHTYPE_HLC		(1 << 3)
#define	PYMOD_GRAPH_GRAPHTYPE_LINEANDBAR	(1 << 4)
#define	PYMOD_GRAPH_GRAPHTYPE_LINEANDAREA	(1 << 5)
#define	PYMOD_GRAPH_GRAPHTYPE_HLCANDBAR		(1 << 6)
#define	PYMOD_GRAPH_GRAPHTYPE_HLCANDAREA	(1 << 7)
#define	PYMOD_GRAPH_GRAPHTYPE_BAR3D		(1 << 8)
#define	PYMOD_GRAPH_GRAPHTYPE_AREA3D		(1 << 9)
#define	PYMOD_GRAPH_GRAPHTYPE_LINE3D		(1 << 10)
#define	PYMOD_GRAPH_GRAPHTYPE_HLC3D		(1 << 11)
#define	PYMOD_GRAPH_GRAPHTYPE_LINEANDBAR3D	(1 << 12)
#define	PYMOD_GRAPH_GRAPHTYPE_LINEANDAREA3D	(1 << 13)
#define	PYMOD_GRAPH_GRAPHTYPE_HLCANDBAR3D	(1 << 14)
#define	PYMOD_GRAPH_GRAPHTYPE_HLCANDAREA3D	(1 << 15)

typedef struct {
	void		(*dtor)(void *);
	void		*(*ctor)(PyObject *,void *,int,int);
	void		(*plot)(void *,void *,int);
} pymodGraphDrivers;


/*****************************************************************************/
/* The Color class.                                                          */
/*****************************************************************************/

typedef struct {
	void		(*dtor)(void *);
	void		*(*ctor)(PyObject *,pymodColorDef *);
	int		(*setRGB)(void *,pymodColorDef *);
} pymodColorDrivers;


void pymodInit(char *,
		pymodColorDrivers *,
		pymodGridDrivers *,
		pymodHBoxDrivers *,
		pymodVBoxDrivers *,
		pymodPieChartDrivers *,
		pymodGraphDrivers *,
		pymodLabelDrivers *,
		pymodApplicationWindowDrivers *,
		pymodMessageDrivers *,
		pymodDialogDrivers *);

void pymodWidgetDeleted(PyObject *);


#ifdef __cplusplus
}
#endif

#endif
