/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module provides a utility functions for the class implementations.
 */


#include "pymodint.h"


/*
 * Raise an exception after a toolkit ctor returns NULL.
 */

void pymodBadToolkitCtor(char *class)
{
	PyErr_Format(PyExc_RuntimeError,"There was a toolkit error creating an instance of the %s class",class);
}


/*
 * Convert an enum value to a new reference of the corresponding Python object.
 * Return NULL if there was an error.
 */

PyObject *pymodConvertEnum(enumDef *ed,int val)
{
	enumValueDef *evd;
	PyObject *obj;

	for (evd = ed -> values; evd -> name != NULL; ++evd)
		if (evd -> intval == val)
			break;

	if (evd -> name == NULL)
	{
		PyErr_Format(PyExc_SystemError,"Internal error: bad value for %s",ed -> type -> tp_name);
		return NULL;
	}

	Py_INCREF(evd -> objval);
	return evd -> objval;
}


/*
 * The repr() function for all enum types.
 */
 
PyObject *pymodEnumRepr(enumObject *eo)
{
	char buf[100];
 
	sprintf(buf,"vp.%s.%s",eo -> cd -> name,eo -> evd -> name);
 
	return PyString_FromString(buf);
}


/*
 * Let a toolkit tell us it has deleted an object.
 */

void pymodWidgetDeleted(PyObject *wo)
{
	((pymodPyBase *)wo) -> base.tkPtr = NULL;
}


/*
 * Convert a Python object to a double.
 */

int pymodConvertDouble(PyObject *val,double *p)
{
	PyErr_Clear();

	*p = PyFloat_AsDouble(val);

	return (PyErr_Occurred() == NULL ? 0 : -1);
}
